@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 /* DisplayList interface */
 
@@ -134,15 +134,14 @@ FUN(DisplayList_toStructuredText)(JNIEnv *env, jobject self, jstring joptions)
 	return to_StructuredText_safe_own(ctx, env, text);
 }
 
-JNIEXPORT jobject JNICALL
+JNIEXPORT jobjectArray JNICALL
 FUN(DisplayList_search)(JNIEnv *env, jobject self, jstring jneedle)
 {
 	fz_context *ctx = get_context(env);
 	fz_display_list *list = from_DisplayList(env, self);
-	fz_quad hits[500];
-	int marks[500];
 	const char *needle = NULL;
-	int n = 0;
+	search_state state = { env, NULL, 0 };
+	jobject jsample = NULL;
 
 	if (!ctx || !list) return NULL;
 	if (!jneedle) jni_throw_arg(env, "needle must not be null");
@@ -150,14 +149,23 @@ FUN(DisplayList_search)(JNIEnv *env, jobject self, jstring jneedle)
 	needle = (*env)->GetStringUTFChars(env, jneedle, NULL);
 	if (!needle) return NULL;
 
+	state.hits = (*env)->NewObject(env, cls_ArrayList, mid_ArrayList_init);
+	if (!state.hits || (*env)->ExceptionCheck(env)) return NULL;
+
 	fz_try(ctx)
-		n = fz_search_display_list(ctx, list, needle, marks, hits, nelem(hits));
+		fz_search_display_list_cb(ctx, list, needle, hit_callback, &state);
 	fz_always(ctx)
+	{
 		(*env)->ReleaseStringUTFChars(env, jneedle, needle);
+	}
 	fz_catch(ctx)
 		jni_rethrow(env, ctx);
 
-	return to_SearchHits_safe(ctx, env, marks, hits, n);
+	if (state.error)
+		return NULL;
+
+	jsample = (*env)->NewObjectArray(env, 0, cls_ArrayOfQuad, NULL);
+	return (*env)->CallObjectMethod(env, state.hits, mid_ArrayList_toArray, jsample);
 }
 
 JNIEXPORT jobject JNICALL
@@ -176,3 +184,34 @@ FUN(DisplayList_getBounds)(JNIEnv *env, jobject self)
 
 	return to_Rect(ctx, env, bounds);
 }
+
+JNIEXPORT jobject JNICALL
+FUN(DisplayList_decodeBarcode)(JNIEnv *env, jobject self, jobject jsubarea, jfloat rotate)
+{
+	fz_context *ctx = get_context(env);
+	fz_display_list *list = from_DisplayList(env, self);
+	fz_rect subarea = from_Rect(env, jsubarea);
+	fz_barcode_type type = FZ_BARCODE_NONE;
+	char *contents = NULL;
+	jobject jcontents;
+	jobject jbarcodeinfo;
+
+	if (!ctx || !list)
+		return NULL;
+
+	fz_try(ctx)
+		contents = fz_decode_barcode_from_display_list(ctx, &type, list, subarea, rotate);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	jcontents = (*env)->NewStringUTF(env, contents);
+	fz_free(ctx, contents);
+	if (!jcontents || (*env)->ExceptionCheck(env))
+		return NULL;
+
+	jbarcodeinfo = (*env)->NewObject(env, cls_BarcodeInfo, mid_BarcodeInfo_init, type, jcontents);
+	if (!jbarcodeinfo || (*env)->ExceptionCheck(env))
+		return NULL;
+
+	return jbarcodeinfo;
+}
