@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -289,16 +289,19 @@ FUN(Document_openNativeWithStream)(JNIEnv *env, jclass cls, jstring jmagic, jobj
 		jni_throw_run(env, "cannot create internal buffer for document stream");
 	}
 
-	accarray = (*env)->NewByteArray(env, sizeof accstate->buffer);
-	if (accarray)
-		accarray = (*env)->NewGlobalRef(env, accarray);
-	if (!accarray)
+	if (jacc)
 	{
-		(*env)->DeleteGlobalRef(env, docarray);
-		(*env)->DeleteGlobalRef(env, jacc);
-		(*env)->DeleteGlobalRef(env, jdoc);
-		if (magic) (*env)->ReleaseStringUTFChars(env, jmagic, magic);
-		jni_throw_run(env, "cannot create internal buffer for accelerator stream");
+		accarray = (*env)->NewByteArray(env, sizeof accstate->buffer);
+		if (accarray)
+			accarray = (*env)->NewGlobalRef(env, accarray);
+		if (!accarray)
+		{
+			(*env)->DeleteGlobalRef(env, docarray);
+			(*env)->DeleteGlobalRef(env, jacc);
+			(*env)->DeleteGlobalRef(env, jdoc);
+			if (magic) (*env)->ReleaseStringUTFChars(env, jmagic, magic);
+			jni_throw_run(env, "cannot create internal buffer for accelerator stream");
+		}
 	}
 
 	fz_try(ctx)
@@ -1021,25 +1024,31 @@ FUN(Document_search)(JNIEnv *env, jobject self, jint chapter, jint page, jstring
 {
 	fz_context *ctx = get_context(env);
 	fz_document *doc = from_Document(env, self);
-	fz_quad hits[500];
-	int marks[500];
 	const char *needle = NULL;
-	int n = 0;
+	search_state state = { env, NULL, 0 };
 
-	if (!ctx) return NULL;
-	if (!jneedle) return NULL;
+	if (!ctx || !doc) return NULL;
+	if (!jneedle) jni_throw_arg(env, "needle must not be null");
 
 	needle = (*env)->GetStringUTFChars(env, jneedle, NULL);
-	if (!needle) return 0;
+	if (!needle) return NULL;
+
+	state.hits = (*env)->NewObject(env, cls_ArrayList, mid_ArrayList_init);
+	if (!state.hits || (*env)->ExceptionCheck(env)) return NULL;
 
 	fz_try(ctx)
-		n = fz_search_chapter_page_number(ctx, doc, chapter, page, needle, marks, hits, nelem(hits));
+		fz_search_chapter_page_number_cb(ctx, doc, chapter, page, needle, hit_callback, &state);
 	fz_always(ctx)
+	{
 		(*env)->ReleaseStringUTFChars(env, jneedle, needle);
+	}
 	fz_catch(ctx)
 		jni_rethrow(env, ctx);
 
-	return to_SearchHits_safe(ctx, env, marks, hits, n);
+	if (state.error)
+		return NULL;
+
+	return (*env)->CallObjectMethod(env, state.hits, mid_ArrayList_toArray);
 }
 
 JNIEXPORT jobject JNICALL
