@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 /* Page interface */
 
@@ -57,7 +57,7 @@ FUN(Page_toPixmap)(JNIEnv *env, jobject self, jobject jctm, jobject jcs, jboolea
 }
 
 JNIEXPORT jobject JNICALL
-FUN(Page_getBounds)(JNIEnv *env, jobject self)
+FUN(Page_getBoundsNative)(JNIEnv *env, jobject self, jint box)
 {
 	fz_context *ctx = get_context(env);
 	fz_page *page = from_Page(env, self);
@@ -66,7 +66,7 @@ FUN(Page_getBounds)(JNIEnv *env, jobject self)
 	if (!ctx || !page) return NULL;
 
 	fz_try(ctx)
-		rect = fz_bound_page(ctx, page);
+		rect = fz_bound_page_box(ctx, page, box);
 	fz_catch(ctx)
 		jni_rethrow(env, ctx);
 
@@ -219,32 +219,14 @@ FUN(Page_getLinks)(JNIEnv *env, jobject self)
 	link = links;
 	for (i = 0; link && i < link_count; i++)
 	{
-		jobject jbounds = NULL;
 		jobject jlink = NULL;
-		jobject juri = NULL;
 
-		jbounds = to_Rect_safe(ctx, env, link->rect);
-		if (!jbounds || (*env)->ExceptionCheck(env))
-		{
-			fz_drop_link(ctx, links);
-			return NULL;
-		}
-
-		juri = (*env)->NewStringUTF(env, link->uri);
-		if (!juri || (*env)->ExceptionCheck(env))
-		{
-			fz_drop_link(ctx, links);
-			return NULL;
-		}
-
-		jlink = (*env)->NewObject(env, cls_Link, mid_Link_init, jbounds, juri);
+		jlink = to_Link_safe(ctx, env, link);
 		if (!jlink || (*env)->ExceptionCheck(env))
 		{
 			fz_drop_link(ctx, links);
 			return NULL;
 		}
-		(*env)->DeleteLocalRef(env, juri);
-		(*env)->DeleteLocalRef(env, jbounds);
 
 		(*env)->SetObjectArrayElement(env, jlinks, i, jlink);
 		if ((*env)->ExceptionCheck(env))
@@ -262,30 +244,38 @@ FUN(Page_getLinks)(JNIEnv *env, jobject self)
 	return jlinks;
 }
 
-JNIEXPORT jobject JNICALL
+JNIEXPORT jobjectArray JNICALL
 FUN(Page_search)(JNIEnv *env, jobject self, jstring jneedle)
 {
 	fz_context *ctx = get_context(env);
 	fz_page *page = from_Page(env, self);
-	fz_quad hits[500];
-	int marks[500];
 	const char *needle = NULL;
-	int n = 0;
+	search_state state = { env, NULL, 0 };
+	jobject jsample = NULL;
 
 	if (!ctx || !page) return NULL;
-	if (!jneedle) return NULL;
+	if (!jneedle) jni_throw_arg(env, "needle must not be null");
 
 	needle = (*env)->GetStringUTFChars(env, jneedle, NULL);
-	if (!needle) return 0;
+	if (!needle) return NULL;
+
+	state.hits = (*env)->NewObject(env, cls_ArrayList, mid_ArrayList_init);
+	if (!state.hits || (*env)->ExceptionCheck(env)) return NULL;
 
 	fz_try(ctx)
-		n = fz_search_page(ctx, page, needle, marks, hits, nelem(hits));
+		fz_search_page_cb(ctx, page, needle, hit_callback, &state);
 	fz_always(ctx)
+	{
 		(*env)->ReleaseStringUTFChars(env, jneedle, needle);
+	}
 	fz_catch(ctx)
 		jni_rethrow(env, ctx);
 
-	return to_SearchHits_safe(ctx, env, marks, hits, n);
+	if (state.error)
+		return NULL;
+
+	jsample = (*env)->NewObjectArray(env, 0, cls_ArrayOfQuad, NULL);
+	return (*env)->CallObjectMethod(env, state.hits, mid_ArrayList_toArray, jsample);
 }
 
 JNIEXPORT jobject JNICALL
@@ -437,6 +427,22 @@ FUN(Page_createLink)(JNIEnv *env, jobject self, jobject jrect, jstring juri)
 	return to_Link_safe_own(ctx, env, link);
 }
 
+JNIEXPORT void JNICALL
+FUN(Page_deleteLink)(JNIEnv *env, jobject self, jobject jlink)
+{
+	fz_context *ctx = get_context(env);
+	fz_page *page = from_Page(env, self);
+	fz_link *link = from_Link(env, jlink);
+
+	if (!ctx || !page)
+		return;
+
+	fz_try(ctx)
+		fz_delete_link(ctx, page, link);
+	fz_catch(ctx)
+		jni_rethrow_void(env, ctx);
+}
+
 JNIEXPORT jobject JNICALL
 FUN(Page_getDocument)(JNIEnv *env, jobject self)
 {
@@ -448,3 +454,52 @@ FUN(Page_getDocument)(JNIEnv *env, jobject self)
 
 	return to_Document_safe_own(ctx, env, fz_keep_document(ctx, doc));
 }
+
+JNIEXPORT jstring JNICALL
+FUN(Page_getLabel)(JNIEnv *env, jobject self)
+{
+	fz_context *ctx = get_context(env);
+	fz_page *page = from_Page(env, self);
+	char buf[100];
+
+	if (!ctx || !page)
+		return NULL;
+
+	fz_try(ctx)
+		fz_page_label(ctx, page, buf, sizeof buf);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	return (*env)->NewStringUTF(env, buf);
+}
+
+JNIEXPORT jobject JNICALL
+FUN(Page_decodeBarcode)(JNIEnv *env, jobject self, jobject jsubarea, jfloat rotate)
+{
+	fz_context *ctx = get_context(env);
+	fz_page *page = from_Page(env, self);
+	fz_rect subarea = from_Rect(env, jsubarea);
+	fz_barcode_type type = FZ_BARCODE_NONE;
+	char *contents = NULL;
+	jobject jcontents;
+	jobject jbarcodeinfo;
+
+	if (!ctx || !page)
+		return NULL;
+
+	fz_try(ctx)
+		contents = fz_decode_barcode_from_page(ctx, &type, page, subarea, rotate);
+	fz_catch(ctx)
+		jni_rethrow(env, ctx);
+
+	jcontents = (*env)->NewStringUTF(env, contents);
+	fz_free(ctx, contents);
+	if (!jcontents || (*env)->ExceptionCheck(env))
+		return NULL;
+
+	jbarcodeinfo = (*env)->NewObject(env, cls_BarcodeInfo, mid_BarcodeInfo_init, type, jcontents);
+	if (!jbarcodeinfo || (*env)->ExceptionCheck(env))
+		return NULL;
+
+	return jbarcodeinfo;
+}
