@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2025 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 package com.artifex.mupdf.fitz;
 
 public class Document
@@ -10,6 +32,12 @@ public class Document
 	public static final String META_ENCRYPTION = "encryption";
 	public static final String META_INFO_AUTHOR = "info:Author";
 	public static final String META_INFO_TITLE = "info:Title";
+	public static final String META_INFO_SUBJECT = "info:Subject";
+	public static final String META_INFO_KEYWORDS = "info:Keywords";
+	public static final String META_INFO_CREATOR = "info:Creator";
+	public static final String META_INFO_PRODUCER = "info:Producer";
+	public static final String META_INFO_CREATIONDATE = "info:CreationDate";
+	public static final String META_INFO_MODIFICATIONDATE = "info:ModDate";
 
 	protected long pointer;
 
@@ -58,6 +86,7 @@ public class Document
 
 	public static native boolean recognize(String magic);
 
+	public native boolean supportsAccelerator();
 	public native void saveAccelerator(String filename);
 	public native void outputAccelerator(SeekableOutputStream stream);
 
@@ -163,18 +192,31 @@ public class Document
 		return -1;
 	}
 
-	public native Quad[] search(int chapter, int page, String needle);
+	public native Quad[][] search(int chapter, int page, String needle);
 
 	public native Location resolveLink(String uri);
 	public Location resolveLink(Outline link) {
 		return resolveLink(link.uri);
 	}
 	public Location resolveLink(Link link) {
-		return resolveLink(link.uri);
+		return resolveLink(link.getURI());
+	}
+
+	public native LinkDestination resolveLinkDestination(String uri);
+	public LinkDestination resolveLinkDestination(OutlineIterator.OutlineItem item) {
+		return resolveLinkDestination(item.uri);
+	}
+	public LinkDestination resolveLinkDestination(Outline link) {
+		return resolveLinkDestination(link.uri);
+	}
+	public LinkDestination resolveLinkDestination(Link link) {
+		return resolveLinkDestination(link.getURI());
 	}
 
 	public native Outline[] loadOutline();
+	public native OutlineIterator outlineIterator();
 	public native String getMetaData(String key);
+	public native void setMetaData(String key, String value);
 	public native boolean isReflowable();
 	public native void layout(float width, float height, float em);
 
@@ -188,12 +230,20 @@ public class Document
 	public static final int PERMISSION_COPY = (int) 'c';
 	public static final int PERMISSION_EDIT = (int) 'e';
 	public static final int PERMISSION_ANNOTATE = (int) 'n';
+	public static final int PERMISSION_FORM = (int) 'f';
+	public static final int PERMISSION_ACCESSIBILITY = (int) 'y';
+	public static final int PERMISSION_ASSEMBLE = (int) 'a';
+	public static final int PERMISSION_PRINT_HQ = (int) 'h';
 
 	public native boolean hasPermission(int permission);
 
 	public native boolean isUnencryptedPDF();
 
+	public native String formatLinkURI(LinkDestination dest);
+
 	public boolean isPDF() {
 		return false;
 	}
+
+	public native PDFDocument asPDF();
 }
