@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2022 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 package com.artifex.mupdf.fitz;
 
@@ -28,7 +28,14 @@ public class TreeArchive extends Archive
 		Context.init();
 	}
 
-	private TreeArchive(long p) { super(p); }
+	private static native long newNativeTreeArchive();
+	private TreeArchive(long p) {
+		super(p);
+	}
+	public TreeArchive()
+	{
+		this(newNativeTreeArchive());
+	}
 
 	public native void add(String name, Buffer buf);
 }
