@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -127,6 +127,19 @@ fz_open_image_decomp_stream(fz_context *ctx, fz_stream *tail, fz_compression_par
 			}
 			break;
 
+		case FZ_IMAGE_BROTLI:
+			head = fz_open_brotlid(ctx, tail);
+			if (params->u.brotli.predictor > 1)
+			{
+				body = head;
+				head = fz_open_predict(ctx, body,
+						params->u.brotli.predictor,
+						params->u.brotli.columns,
+						params->u.brotli.colors,
+						params->u.brotli.bpc);
+			}
+			break;
+
 		case FZ_IMAGE_LZW:
 			head = fz_open_lzwd(ctx, tail, params->u.lzw.early_change, 9, 0, 0);
 			if (params->u.flate.predictor > 1)
