@@ -98,15 +98,15 @@ next_jbig2d(fz_context *ctx, fz_stream *stm, size_t len)
 				break;
 
 			if (jbig2_data_in(state->ctx, tmp, n) < 0)
-				fz_throw(ctx, FZ_ERROR_GENERIC, "cannot decode jbig2 image");
+				fz_throw(ctx, FZ_ERROR_LIBRARY, "cannot decode jbig2 image");
 		}
 
 		if (jbig2_complete_page(state->ctx) < 0)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "cannot complete jbig2 image");
+			fz_throw(ctx, FZ_ERROR_LIBRARY, "cannot complete jbig2 image");
 
 		state->page = jbig2_page_out(state->ctx);
 		if (!state->page)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "no jbig2 image decoded");
+			fz_throw(ctx, FZ_ERROR_LIBRARY, "no jbig2 image decoded");
 	}
 
 	s = state->page->data;
@@ -168,9 +168,14 @@ static void *fz_jbig2_realloc(Jbig2Allocator *allocator, void *p, size_t size)
 fz_jbig2_globals *
 fz_load_jbig2_globals(fz_context *ctx, fz_buffer *buf)
 {
-	fz_jbig2_globals *globals = fz_malloc_struct(ctx, fz_jbig2_globals);
+	fz_jbig2_globals *globals;
 	Jbig2Ctx *jctx;
 
+	if (buf == NULL || buf->data == NULL || buf->len == 0)
+		return NULL;
+
+	globals = fz_malloc_struct(ctx, fz_jbig2_globals);
+
 	globals->alloc.ctx = ctx;
 	globals->alloc.alloc.alloc = fz_jbig2_alloc;
 	globals->alloc.alloc.free = fz_jbig2_free;
@@ -180,14 +185,14 @@ fz_load_jbig2_globals(fz_context *ctx, fz_buffer *buf)
 	if (!jctx)
 	{
 		fz_free(ctx, globals);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot allocate jbig2 globals context");
+		fz_throw(ctx, FZ_ERROR_LIBRARY, "cannot allocate jbig2 globals context");
 	}
 
 	if (jbig2_data_in(jctx, buf->data, buf->len) < 0)
 	{
 		jbig2_global_ctx_free(jbig2_make_global_ctx(jctx));
 		fz_free(ctx, globals);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot decode jbig2 globals");
+		fz_throw(ctx, FZ_ERROR_LIBRARY, "cannot decode jbig2 globals");
 	}
 
 	FZ_INIT_STORABLE(globals, 1, fz_drop_jbig2_globals_imp);
@@ -232,7 +237,7 @@ fz_open_jbig2d(fz_context *ctx, fz_stream *chain, fz_jbig2_globals *globals, int
 	{
 		fz_drop_jbig2_globals(ctx, state->gctx);
 		fz_free(ctx, state);
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot allocate jbig2 context");
+		fz_throw(ctx, FZ_ERROR_LIBRARY, "cannot allocate jbig2 context");
 	}
 
 	state->page = NULL;
