@@ -1,4 +1,31 @@
-#include "fitz-imp.h"
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
+#include "mupdf/fitz.h"
+
+#include "pixmap-imp.h"
+
+#include <string.h>
+#include <limits.h>
 
 struct info
 {
@@ -23,6 +50,8 @@ struct info
 	int has_transparency;
 	unsigned int transparent;
 	unsigned char *mask;
+
+	fz_pixmap *pix;
 };
 
 /* default color table, where the first two entries are black and white */
@@ -93,24 +122,24 @@ static const unsigned char dct[256 * 3] = {
 	0xfb, 0xfb, 0xfb, 0xfc, 0xfc, 0xfc, 0xfd, 0xfd, 0xfd, 0xfe, 0xfe, 0xfe,
 };
 
-static unsigned char *
-gif_read_subblocks(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end, fz_buffer *buf)
+static const unsigned char *
+gif_read_subblocks(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end, fz_buffer *buf)
 {
 	int len;
 
 	do
 	{
 		if (end - p < 1)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in data subblocks in gif image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in data subblocks in gif image");
 		len = *p;
 		p += 1;
 
 		if (len > 0)
 		{
 			if (end - p < len)
-				fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in data subblock in gif image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in data subblock in gif image");
 			if (buf)
-				fz_write_buffer(ctx, buf, p, len);
+				fz_append_data(ctx, buf, p, len);
 			p += len;
 		}
 	} while (len > 0);
@@ -118,36 +147,43 @@ gif_read_subblocks(fz_context *ctx, struct info *info, unsigned char *p, unsigne
 	return p;
 }
 
-static unsigned char *
-gif_read_header(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_header(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < 6)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in header in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in header in gif image");
 
 	if (memcmp(&p[0], "GIF", 3))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "invalid signature in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "invalid signature in gif image");
 	if (memcmp(&p[3], "87a", 3) && memcmp(&p[3], "89a", 3))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported version in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported version in gif image");
 
 	info->gif89a = !memcmp(p, "GIF89a", 6);
 
 	return p + 6;
 }
 
-static unsigned char *
-gif_read_lsd(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+/* coverity[-tainted_data_return] */
+static unsigned int
+safe_load_u16(const unsigned char *p)
+{
+	return p[1] << 8 | p[0];
+}
+
+static const unsigned char *
+gif_read_lsd(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < 7)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in logical screen descriptor in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in logical screen descriptor in gif image");
 
-	info->width = p[1] << 8 | p[0];
-	info->height = p[3] << 8 | p[2];
+	info->width = safe_load_u16(p);
+	info->height = safe_load_u16(p+2);
 	if (info->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if (info->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (info->height > UINT_MAX / info->width / 3 /* components */)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image dimensions might overflow");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image dimensions might overflow");
 
 	info->has_gct = (p[4] >> 7) & 0x1;
 	if (info->has_gct)
@@ -158,30 +194,30 @@ gif_read_lsd(fz_context *ctx, struct info *info, unsigned char *p, unsigned char
 	info->aspect = p[6];
 
 	info->xres = 96;
-	info->yres= 96;
+	info->yres = 96;
 	if (info->aspect > 0)
 		info->yres = (((float) info->aspect + 15) / 64) * 96;
 
 	return p + 7;
 }
 
-static unsigned char *
-gif_read_gct(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_gct(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < info->gct_entries * 3)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in global color table in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in global color table in gif image");
 
-	info->gct = fz_malloc(ctx, info->gct_entries * 3);
+	info->gct = Memento_label(fz_malloc(ctx, info->gct_entries * 3), "gif_gct");
 	memmove(info->gct, p, info->gct_entries * 3);
 
 	return p + info->gct_entries * 3;
 }
 
-static unsigned char *
-gif_read_id(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_id(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < 10)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in image descriptor in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in image descriptor in gif image");
 
 	info->image_left = p[2] << 8 | p[1];
 	info->image_top = p[4] << 8 | p[3];
@@ -196,23 +232,24 @@ gif_read_id(fz_context *ctx, struct info *info, unsigned char *p, unsigned char
 	return p + 10;
 }
 
-static unsigned char *
-gif_read_lct(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_lct(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < info->lct_entries * 3)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in local color table in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in local color table in gif image");
 
-	info->lct = fz_malloc(ctx, info->lct_entries * 3);
+	info->lct = Memento_label(fz_malloc(ctx, info->lct_entries * 3), "gif_lct");
 	memmove(info->lct, p, info->lct_entries * 3);
 
 	return p + info->lct_entries * 3;
 }
 
 static void
-gif_read_line(fz_context *ctx, struct info *info, unsigned char *dest, int ct_entries, const unsigned char *ct, unsigned int y, unsigned char *sp)
+gif_read_line(fz_context *ctx, struct info *info, int ct_entries, const unsigned char *ct, unsigned int y, unsigned char *sp)
 {
 	unsigned int index = (info->image_top + y) * info->width + info->image_left;
-	unsigned char *dp = &dest[index * 4];
+	unsigned char *samples = fz_pixmap_samples(ctx, info->pix);
+	unsigned char *dp = &samples[index * 4];
 	unsigned char *mp = &info->mask[index];
 	unsigned int x, k;
 
@@ -231,10 +268,10 @@ gif_read_line(fz_context *ctx, struct info *info, unsigned char *dest, int ct_en
 			*mp = 0x00;
 }
 
-static unsigned char *
-gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_tbid(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
-	fz_stream *stm, *lzwstm = NULL;
+	fz_stream *stm = NULL, *lzwstm = NULL;
 	unsigned int mincodesize, y;
 	fz_buffer *compressed = NULL, *uncompressed = NULL;
 	const unsigned char *ct;
@@ -242,7 +279,7 @@ gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned
 	int ct_entries;
 
 	if (end - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in table based image data in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in table based image data in gif image");
 
 	mincodesize = *p;
 
@@ -255,6 +292,7 @@ gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned
 
 	fz_var(compressed);
 	fz_var(lzwstm);
+	fz_var(stm);
 	fz_var(uncompressed);
 
 	fz_try(ctx)
@@ -263,11 +301,15 @@ gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned
 		p = gif_read_subblocks(ctx, info, p + 1, end, compressed);
 
 		stm = fz_open_buffer(ctx, compressed);
-		lzwstm = fz_open_lzwd(ctx, stm, 0, mincodesize + 1, 1, 0);
+		lzwstm = fz_open_lzwd(ctx, stm, 0, mincodesize + 1, 1, 1);
 
 		uncompressed = fz_read_all(ctx, lzwstm, 0);
-		if (uncompressed->len < info->image_width * info->image_height)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in compressed table based image data in gif image");
+		if (uncompressed->len < (size_t)info->image_width * info->image_height)
+		{
+			fz_warn(ctx, "premature end in compressed table based image data in gif image");
+			while (uncompressed->len < (size_t)info->image_width * info->image_height)
+				fz_append_byte(ctx, uncompressed, 0x00);
+		}
 
 		if (info->has_lct)
 		{
@@ -289,23 +331,24 @@ gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned
 		if (info->image_interlaced)
 		{
 			for (y = 0; y < info->image_height; y += 8, sp += info->image_width)
-				gif_read_line(ctx, info, dest, ct_entries, ct, y, sp);
+				gif_read_line(ctx, info, ct_entries, ct, y, sp);
 			for (y = 4; y < info->image_height; y += 8, sp += info->image_width)
-				gif_read_line(ctx, info, dest, ct_entries, ct, y, sp);
+				gif_read_line(ctx, info, ct_entries, ct, y, sp);
 			for (y = 2; y < info->image_height; y += 4, sp += info->image_width)
-				gif_read_line(ctx, info, dest, ct_entries, ct, y, sp);
+				gif_read_line(ctx, info, ct_entries, ct, y, sp);
 			for (y = 1; y < info->image_height; y += 2, sp += info->image_width)
-				gif_read_line(ctx, info, dest, ct_entries, ct, y, sp);
+				gif_read_line(ctx, info, ct_entries, ct, y, sp);
 		}
 		else
 			for (y = 0; y < info->image_height; y++, sp += info->image_width)
-				gif_read_line(ctx, info, dest, ct_entries, ct, y, sp);
+				gif_read_line(ctx, info, ct_entries, ct, y, sp);
 	}
 	fz_always(ctx)
 	{
 		fz_drop_buffer(ctx, uncompressed);
 		fz_drop_buffer(ctx, compressed);
 		fz_drop_stream(ctx, lzwstm);
+		fz_drop_stream(ctx, stm);
 	}
 	fz_catch(ctx)
 	{
@@ -315,13 +358,13 @@ gif_read_tbid(fz_context *ctx, struct info *info, unsigned char *dest, unsigned
 	return p;
 }
 
-static unsigned char *
-gif_read_gce(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_gce(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < 8)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in graphic control extension in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in graphic control extension in gif image");
 	if (p[2] != 0x04)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "out of range graphic control extension block size in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "out of range graphic control extension block size in gif image");
 
 	info->has_transparency = p[3] & 0x1;
 	if (info->has_transparency)
@@ -330,22 +373,54 @@ gif_read_gce(fz_context *ctx, struct info *info, unsigned char *p, unsigned char
 	return p + 8;
 }
 
-static unsigned char *
-gif_read_ce(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_ce(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	return gif_read_subblocks(ctx, info, p + 2, end, NULL);
 }
 
-static unsigned char*
-gif_read_pte(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char*
+gif_read_pte(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	if (end - p < 15)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in plain text extension in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in plain text extension in gif image");
 	if (p[2] != 0x0c)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "out of range plain text extension block size in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "out of range plain text extension block size in gif image");
 	return gif_read_subblocks(ctx, info, p + 15, end, NULL);
 }
 
+static const unsigned char *
+gif_read_icc(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
+{
+#if FZ_ENABLE_ICC
+	fz_colorspace *icc = NULL;
+	fz_buffer *buf = NULL;
+
+	fz_var(p);
+
+	buf = fz_new_buffer(ctx, 0);
+	fz_try(ctx)
+	{
+		p = gif_read_subblocks(ctx, info, p, end, buf);
+		icc = fz_new_icc_colorspace(ctx, FZ_COLORSPACE_RGB, 0, NULL, buf);
+		fz_drop_colorspace(ctx, info->pix->colorspace);
+		info->pix->colorspace = icc;
+	}
+	fz_always(ctx)
+		fz_drop_buffer(ctx, buf);
+	fz_catch(ctx)
+	{
+		fz_rethrow_if(ctx, FZ_ERROR_SYSTEM);
+		fz_report_error(ctx);
+		fz_warn(ctx, "ignoring embedded ICC profile in GIF");
+	}
+
+	return p;
+#else
+	return gif_read_subblocks(ctx, info, p, end, NULL);
+#endif
+}
+
 /*
 NETSCAPE2.0
 	http://odur.let.rug.nl/~kleiweg/gif/netscape.html
@@ -371,23 +446,23 @@ ZGATEXTI5 ZGATILEI5 ZGACTRLI5 ZGANPIMGI5
 ZGAVECTI5 ZGAALPHAI5 ZGATITLE4.0 ZGATEXTI4.0
 	Zoner GIF animator 4.0 and 5.0
 */
-static unsigned char *
-gif_read_ae(fz_context *ctx, struct info *info, unsigned char *p, unsigned char *end)
+static const unsigned char *
+gif_read_ae(fz_context *ctx, struct info *info, const unsigned char *p, const unsigned char *end)
 {
 	static char *ignorable[] = {
-		"NETSACPE2.0", "ANIMEXTS1.0", "ICCRGBG1012", "XMP DataXMP",
+		"NETSACPE2.0", "ANIMEXTS1.0", "XMP DataXMP",
 		"ZGATEXTI5\0\0", "ZGATILEI5\0\0", "ZGANPIMGI5\0", "ZGACTRLI5\0\0",
 		"ZGAVECTI5\0\0", "ZGAALPHAI5\0", "ZGATITLE4.0", "ZGATEXTI4.0",
 	};
 	int i, ignored;
 
 	if (end - p < 14)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in application extension in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in application extension in gif image");
 	if (p[2] != 0x0b)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "out of range application extension block size in gif image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "out of range application extension block size in gif image");
 
 	ignored = 0;
-	for (i = 0; i < nelem(ignorable); i++)
+	for (i = 0; i < (int)nelem(ignorable); i++)
 		ignored |= memcmp(&p[3], ignorable[i], 8 + 3);
 	if (!ignored)
 	{
@@ -397,27 +472,29 @@ gif_read_ae(fz_context *ctx, struct info *info, unsigned char *p, unsigned char
 		fz_warn(ctx, "ignoring unsupported application extension '%s' in gif image", extension);
 	}
 
+	if (!memcmp(&p[3], "ICCRGBG1012", 11))
+		return gif_read_icc(ctx, info, p + 14, end);
+
 	return gif_read_subblocks(ctx, info, p + 14, end, NULL);
 }
 
 static void
-gif_mask_transparency(fz_context *ctx, fz_pixmap *image, struct info *info)
+gif_mask_transparency(fz_context *ctx, struct info *info)
 {
 	unsigned char *mp = info->mask;
-	unsigned char *dp = image->samples;
+	unsigned char *dp = fz_pixmap_samples(ctx, info->pix);
 	unsigned int x, y;
 
 	for (y = 0; y < info->height; y++)
-		for (x = 0; x < info->width; x++, mp++, dp += image->n)
+		for (x = 0; x < info->width; x++, mp++, dp += 4)
 			if (*mp == 0x00)
-				dp[image->n - 1] = 0;
+				dp[3] = 0;
 }
 
 static fz_pixmap *
-gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t total, int only_metadata)
+gif_read_image(fz_context *ctx, struct info *info, const unsigned char *p, size_t total, int only_metadata)
 {
-	fz_pixmap *pix;
-	unsigned char *end = p + total;
+	const unsigned char *end = p + total;
 
 	memset(info, 0x00, sizeof (*info));
 
@@ -430,22 +507,22 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 	if (only_metadata)
 		return NULL;
 
-	pix = fz_new_pixmap(ctx, fz_device_rgb(ctx), info->width, info->height, 1);
+	info->pix = fz_new_pixmap(ctx, fz_device_rgb(ctx), info->width, info->height, NULL, 1);
 
 	fz_try(ctx)
 	{
-		info->mask = fz_calloc(ctx, info->width * info->height, 1);
+		info->mask = fz_calloc(ctx, (size_t)info->width * info->height, 1);
 
 		/* Read optional global color table */
 		if (info->has_gct)
 		{
-			unsigned char *bp, *dp = pix->samples;
+			unsigned char *bp, *dp = fz_pixmap_samples(ctx, info->pix);
 			unsigned int x, y, k;
 
 			p = gif_read_gct(ctx, info, p, end);
 			bp = &info->gct[info->gct_background * 3];
 
-			memset(info->mask, 0x01, info->width * info->height);
+			memset(info->mask, 0x01, (size_t)info->width * info->height);
 
 			for (y = 0; y < info->height; y++)
 				for (x = 0; x < info->width; x++, dp += 4)
@@ -460,7 +537,7 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 		{
 			/* Read block indicator */
 			if (end - p < 1)
-				fz_throw(ctx, FZ_ERROR_GENERIC, "premature end of block indicator in gif image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "premature end of block indicator in gif image");
 
 			/* Read trailer */
 			if (p[0] == 0x3b)
@@ -472,7 +549,7 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 			{
 				/* Read extension label */
 				if (end - p < 2)
-					fz_throw(ctx, FZ_ERROR_GENERIC, "premature end in extension label in gif image");
+					fz_throw(ctx, FZ_ERROR_FORMAT, "premature end in extension label in gif image");
 
 				if (p[1] == 0x01 && info->gif89a)
 				{
@@ -493,7 +570,10 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 					/* Read application extension */
 					p = gif_read_ae(ctx, info, p, end);
 				else
-					fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported extension label %02x in gif image", p[1]);
+				{
+					fz_warn(ctx, "ignoring unsupported extension label %02x in gif image", p[1]);
+					p = gif_read_subblocks(ctx, info, p + 2, end, NULL);
+				}
 			}
 			/* Read image descriptor */
 			else if (p[0] == 0x2c)
@@ -505,7 +585,7 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 					p = gif_read_lct(ctx, info, p, end);
 
 				/* Read table based image data */
-				p = gif_read_tbid(ctx, info, pix->samples, p, end);
+				p = gif_read_tbid(ctx, info, p, end);
 
 				/* Graphic control extension applies only to the graphic rendering block following it */
 				info->transparent = 0;
@@ -521,11 +601,11 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 				info->has_lct = 0;
 			}
 			else
-				fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported block indicator %02x in gif image", p[0]);
+				fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported block indicator %02x in gif image", p[0]);
 		}
 
-		gif_mask_transparency(ctx, pix, info);
-		fz_premultiply_pixmap(ctx, pix);
+		gif_mask_transparency(ctx, info);
+		fz_premultiply_pixmap(ctx, info->pix);
 	}
 	fz_always(ctx)
 	{
@@ -535,15 +615,15 @@ gif_read_image(fz_context *ctx, struct info *info, unsigned char *p, size_t tota
 	}
 	fz_catch(ctx)
 	{
-		fz_drop_pixmap(ctx, pix);
+		fz_drop_pixmap(ctx, info->pix);
 		fz_rethrow(ctx);
 	}
 
-	return pix;
+	return info->pix;
 }
 
 fz_pixmap *
-fz_load_gif(fz_context *ctx, unsigned char *p, size_t total)
+fz_load_gif(fz_context *ctx, const unsigned char *p, size_t total)
 {
 	fz_pixmap *image;
 	struct info gif;
@@ -556,13 +636,12 @@ fz_load_gif(fz_context *ctx, unsigned char *p, size_t total)
 }
 
 void
-fz_load_gif_info(fz_context *ctx, unsigned char *p, size_t total, int *wp, int *hp, int *xresp, int *yresp, fz_colorspace **cspacep)
+fz_load_gif_info(fz_context *ctx, const unsigned char *p, size_t total, int *wp, int *hp, int *xresp, int *yresp, fz_colorspace **cspacep)
 {
 	struct info gif;
 
 	gif_read_image(ctx, &gif, p, total, 1);
-
-	*cspacep = fz_device_rgb(ctx);
+	*cspacep = fz_keep_colorspace(ctx, fz_device_rgb(ctx));
 	*wp = gif.width;
 	*hp = gif.height;
 	*xresp = gif.xres;
