@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2025 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 
 #include "pixmap-imp.h"
@@ -29,6 +51,13 @@ enum
 	TOKEN_ENDHDR,
 };
 
+enum
+{
+	ENDIAN_UNKNOWN = 0,
+	ENDIAN_LITTLE,
+	ENDIAN_BIG,
+};
+
 struct info
 {
 	int subimages;
@@ -37,6 +66,8 @@ struct info
 	int maxval, bitdepth;
 	int depth, alpha;
 	int tupletype;
+	int endian;
+	float scale;
 };
 
 static inline int iswhiteeol(int a)
@@ -57,15 +88,6 @@ static inline int iswhite(int a)
 	return 0;
 }
 
-static inline int iseol(int a)
-{
-	switch (a) {
-	case '\r': case '\n':
-		return 1;
-	}
-	return 0;
-}
-
 static inline int bitdepth_from_maxval(int maxval)
 {
 	int depth = 0;
@@ -77,58 +99,13 @@ static inline int bitdepth_from_maxval(int maxval)
 	return depth;
 }
 
-static const unsigned char *
-pnm_read_white(fz_context *ctx, const unsigned char *p, const unsigned char *e, int single_line)
-{
-	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse whitespace in pnm image");
-
-	if (single_line)
-	{
-		if (!iswhiteeol(*p) && *p != '#')
-			fz_throw(ctx, FZ_ERROR_GENERIC, "expected whitespace/comment in pnm image");
-		while (p < e && iswhite(*p))
-			p++;
-
-		if (p < e && *p == '#')
-			while (p < e && !iseol(*p))
-				p++;
-		if (p < e && iseol(*p))
-			p++;
-	}
-	else
-	{
-		if (!iswhiteeol(*p) && *p != '#')
-			fz_throw(ctx, FZ_ERROR_GENERIC, "expected whitespace in pnm image");
-		while (p < e && iswhiteeol(*p))
-			p++;
-
-		while (p < e && *p == '#')
-		{
-			while (p < e && !iseol(*p))
-				p++;
-
-			if (p < e && iseol(*p))
-				p++;
-
-			while (p < e && iswhiteeol(*p))
-				p++;
-
-			if (p < e && iseol(*p))
-				p++;
-		}
-	}
-
-	return p;
-}
-
 static const unsigned char *
 pnm_read_signature(fz_context *ctx, const unsigned char *p, const unsigned char *e, char *signature)
 {
 	if (e - p < 2)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse magic number in pnm image");
-	if (p[0] != 'P' || p[1] < '1' || p[1] > '7')
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected signature in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse magic number in pnm image");
+	if (p[0] != 'P' || ((p[1] < '1' || p[1] > '7') && p[1] != 'F' && p[1] != 'f'))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected signature in pnm image");
 
 	signature[0] = *p++;
 	signature[1] = *p++;
@@ -136,12 +113,122 @@ pnm_read_signature(fz_context *ctx, const unsigned char *p, const unsigned char
 }
 
 static const unsigned char *
-pnm_read_number(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *number)
+pnm_read_until_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse number in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse line in pnm image");
+
+	while (p < e && ((acceptCR && *p != '\r' && *p != '\n') || (!acceptCR && *p != '\n')))
+		p++;
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse end of line in pnm image");
+	if ((acceptCR && *p != '\r' && *p != '\n') || (!acceptCR && *p != '\n'))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected end of line in pnm image");
+
+	/* CR, CRLF or LF depending on acceptCR. */
+	if (acceptCR && *p == '\r')
+		p++;
+	if (p < e && *p == '\n')
+		p++;
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_whites(fz_context *ctx, const unsigned char *p, const unsigned char *e, int required)
+{
+	if (required && e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespaces in pnm image");
+	if (required && !iswhite(*p))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespaces in pnm image");
+
+	while (p < e && iswhite(*p))
+		p++;
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_white_or_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespace/eol in pnm image");
+	if (!iswhiteeol(*p))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespace/eol in pnm image");
+
+	return ++p;
+}
+
+static const unsigned char *
+pnm_read_whites_and_eols(fz_context *ctx, const unsigned char *p, const unsigned char *e, int required)
+{
+	if (required && e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespaces/eols in pnm image");
+	if (required && !iswhiteeol(*p))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespaces/eols in pnm image");
+
+	while (p < e && iswhiteeol(*p))
+		p++;
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_comment(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse line in pnm image");
+
+	if (*p != '#')
+		return p;
+
+	return pnm_read_until_eol(ctx, p, e, acceptCR);
+}
+
+static const unsigned char *
+pnm_read_comments(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse comment in pnm image");
+
+	while (p < e && *p == '#')
+	{
+		p = pnm_read_comment(ctx, p, e, acceptCR);
+		p = pnm_read_eol(ctx, p, e, acceptCR);
+	}
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_digit(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *number)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse digit in pnm image");
+	if (*p < '0' || *p > '1')
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected digit in pnm image");
+
+	if (number)
+		*number = *p - '0';
+	p++;
+
+	return p;
+}
+
+static const unsigned char *
+pnm_read_int(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *number)
+{
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse integer in pnm image");
 	if (*p < '0' || *p > '9')
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected numeric field in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected integer in pnm image");
 
 	while (p < e && *p >= '0' && *p <= '9')
 	{
@@ -153,6 +240,40 @@ pnm_read_number(fz_context *ctx, const unsigned char *p, const unsigned char *e,
 	return p;
 }
 
+static const unsigned char *
+pnm_read_real(fz_context *ctx, const unsigned char *p, const unsigned char *e, float *number)
+{
+	const unsigned char *orig = p;
+	char *buf, *end;
+	size_t len;
+
+	if (e - p < 1)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse real in pnm image");
+
+	if (*p != '+' && *p != '-' && (*p < '0' || *p > '9'))
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected numeric field in pnm image");
+
+	while (p < e && (*p == '+' || *p == '-' || *p == '.' || (*p >= '0' && *p <= '9')))
+		p++;
+
+	len = p - orig + 1;
+	end = buf = fz_malloc(ctx, len);
+
+	fz_try(ctx)
+	{
+		memcpy(buf, orig, len - 1);
+		buf[len - 1] = '\0';
+		*number = fz_strtof(buf, &end);
+		p = orig + (end - buf);
+	}
+	fz_always(ctx)
+		fz_free(ctx, buf);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+
+	return p;
+}
+
 static const unsigned char *
 pnm_read_tupletype(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *tupletype)
 {
@@ -171,10 +292,10 @@ pnm_read_tupletype(fz_context *ctx, const unsigned char *p, const unsigned char
 	int i, len;
 
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse tuple type in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse tuple type in pnm image");
 
 	s = p;
-	while (!iswhiteeol(*p))
+	while (p < e && !iswhiteeol(*p))
 		p++;
 	len = p - s;
 
@@ -185,7 +306,7 @@ pnm_read_tupletype(fz_context *ctx, const unsigned char *p, const unsigned char
 			return p;
 		}
 
-	fz_throw(ctx, FZ_ERROR_GENERIC, "unknown tuple type in pnm image");
+	fz_throw(ctx, FZ_ERROR_FORMAT, "unknown tuple type in pnm image");
 }
 
 static const unsigned char *
@@ -204,10 +325,10 @@ pnm_read_token(fz_context *ctx, const unsigned char *p, const unsigned char *e,
 	int i, len;
 
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse header token in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse header token in pnm image");
 
 	s = p;
-	while (!iswhiteeol(*p))
+	while (p < e && !iswhiteeol(*p))
 		p++;
 	len = p - s;
 
@@ -218,7 +339,7 @@ pnm_read_token(fz_context *ctx, const unsigned char *p, const unsigned char *e,
 			return p;
 		}
 
-	fz_throw(ctx, FZ_ERROR_GENERIC, "unknown header token in pnm image");
+	fz_throw(ctx, FZ_ERROR_FORMAT, "unknown header token in pnm image");
 }
 
 static int
@@ -233,34 +354,43 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 {
 	fz_pixmap *img = NULL;
 
-	p = pnm_read_number(ctx, p, e, &pnm->width);
-	p = pnm_read_white(ctx, p, e, 0);
+	pnm->width = 0;
+	p = pnm_read_comments(ctx, p, e, 1);
+	p = pnm_read_int(ctx, p, e, &pnm->width);
+	p = pnm_read_whites_and_eols(ctx, p, e, 1);
 
 	if (bitmap)
 	{
-		p = pnm_read_number(ctx, p, e, &pnm->height);
-		p = pnm_read_white(ctx, p, e, 1);
+		pnm->height = 0;
+		p = pnm_read_int(ctx, p, e, &pnm->height);
+		p = pnm_read_whites_and_eols(ctx, p, e, 1);
+
 		pnm->maxval = 1;
 	}
 	else
 	{
-		p = pnm_read_number(ctx, p, e, &pnm->height);
-		p = pnm_read_white(ctx, p, e, 0);
-		p = pnm_read_number(ctx, p, e, &pnm->maxval);
-		p = pnm_read_white(ctx, p, e, 0);
+		pnm->height = 0;
+		p = pnm_read_comments(ctx, p, e, 1);
+		p = pnm_read_int(ctx, p, e, &pnm->height);
+		p = pnm_read_whites_and_eols(ctx, p, e, 1);
+
+		pnm->maxval = 0;
+		p = pnm_read_comments(ctx, p, e, 1);
+		p = pnm_read_int(ctx, p, e, &pnm->maxval);
+		p = pnm_read_white_or_eol(ctx, p, e);
 	}
 
 	if (pnm->maxval <= 0 || pnm->maxval >= 65536)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maximum sample value of out range in pnm image: %d", pnm->maxval);
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maximum sample value of out range in pnm image: %d", pnm->maxval);
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
 
 	if (onlymeta)
 	{
@@ -274,10 +404,11 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		if (bitmap)
 		{
 			for (y = 0; y < h; y++)
-				for (x = -1; x < w; x++)
+				for (x = 0; x < w; x++)
 				{
-					p = pnm_read_number(ctx, p, e, NULL);
-					p = pnm_read_white(ctx, p, e, 0);
+					p = pnm_read_whites_and_eols(ctx, p, e, 0);
+					p = pnm_read_digit(ctx, p, e, NULL);
+					p = pnm_read_whites_and_eols(ctx, p, e, 0);
 				}
 		}
 		else
@@ -286,8 +417,9 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 				for (x = 0; x < w; x++)
 					for (k = 0; k < n; k++)
 					{
-						p = pnm_read_number(ctx, p, e, NULL);
-						p = pnm_read_white(ctx, p, e, 0);
+						p = pnm_read_whites_and_eols(ctx, p, e, 0);
+						p = pnm_read_int(ctx, p, e, NULL);
+						p = pnm_read_whites_and_eols(ctx, p, e, 0);
 					}
 		}
 	}
@@ -311,8 +443,9 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 				for (x = 0; x < w; x++)
 				{
 					int v = 0;
-					p = pnm_read_number(ctx, p, e, &v);
-					p = pnm_read_white(ctx, p, e, 0);
+					p = pnm_read_whites_and_eols(ctx, p, e, 0);
+					p = pnm_read_digit(ctx, p, e, &v);
+					p = pnm_read_whites_and_eols(ctx, p, e, 0);
 					*dp++ = v ? 0x00 : 0xff;
 				}
 			}
@@ -324,8 +457,9 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 					for (k = 0; k < n; k++)
 					{
 						int v = 0;
-						p = pnm_read_number(ctx, p, e, &v);
-						p = pnm_read_white(ctx, p, e, 0);
+						p = pnm_read_whites_and_eols(ctx, p, e, 0);
+						p = pnm_read_int(ctx, p, e, &v);
+						p = pnm_read_whites_and_eols(ctx, p, e, 0);
 						v = fz_clampi(v, 0, pnm->maxval);
 						*dp++ = map_color(ctx, v, pnm->maxval, 255);
 					}
@@ -342,60 +476,76 @@ static fz_pixmap *
 pnm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, const unsigned char *e, int onlymeta, int bitmap, const unsigned char **out)
 {
 	fz_pixmap *img = NULL;
+	size_t span;
+	int n;
+
+	n = fz_colorspace_n(ctx, pnm->cs);
+	assert(n >= 1 && n <= 3);
 
 	pnm->width = 0;
-	p = pnm_read_number(ctx, p, e, &pnm->width);
-	p = pnm_read_white(ctx, p, e, 0);
+	p = pnm_read_comments(ctx, p, e, 1);
+	p = pnm_read_int(ctx, p, e, &pnm->width);
+	p = pnm_read_whites_and_eols(ctx, p, e, 1);
 
 	if (bitmap)
 	{
 		pnm->height = 0;
-		p = pnm_read_number(ctx, p, e, &pnm->height);
-		p = pnm_read_white(ctx, p, e, 1);
+		p = pnm_read_int(ctx, p, e, &pnm->height);
+		p = pnm_read_whites_and_eols(ctx, p, e, 1);
+
 		pnm->maxval = 1;
 	}
 	else
 	{
 		pnm->height = 0;
-		p = pnm_read_number(ctx, p, e, &pnm->height);
-		p = pnm_read_white(ctx, p, e, 0);
+		p = pnm_read_comments(ctx, p, e, 1);
+		p = pnm_read_int(ctx, p, e, &pnm->height);
+		p = pnm_read_whites_and_eols(ctx, p, e, 1);
+
 		pnm->maxval = 0;
-		p = pnm_read_number(ctx, p, e, &pnm->maxval);
-		p = pnm_read_white(ctx, p, e, 1);
+		p = pnm_read_comments(ctx, p, e, 1);
+		p = pnm_read_int(ctx, p, e, &pnm->maxval);
+		p = pnm_read_white_or_eol(ctx, p, e);
 	}
 
 	if (pnm->maxval <= 0 || pnm->maxval >= 65536)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maximum sample value of out range in pnm image: %d", pnm->maxval);
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maximum sample value of out range in pnm image: %d", pnm->maxval);
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
-	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
+	if (pnm->bitdepth == 1)
+	{
+		/* Overly sensitive test, but we can live with it. */
+		if ((size_t)pnm->width > SIZE_MAX / (unsigned int)n)
+			fz_throw(ctx, FZ_ERROR_LIMIT, "image row too large");
+		span = ((size_t)n * pnm->width + 7)/8;
+	}
+	else
+	{
+		size_t bytes_per_sample = (pnm->bitdepth-1)/8 + 1;
+		span = (size_t)n * bytes_per_sample;
+		if ((size_t)pnm->width > SIZE_MAX / span)
+			fz_throw(ctx, FZ_ERROR_LIMIT, "image row too large");
+		span = (size_t)pnm->width * span;
+	}
+	if ((size_t)pnm->height > SIZE_MAX / span)
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
+	if (e - p < 0 || ((size_t)(e - p)) < span * (size_t)pnm->height)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "insufficient data");
 
 	if (onlymeta)
 	{
-		int w = pnm->width;
-		int h = pnm->height;
-		int n = fz_colorspace_n(ctx, pnm->cs);
-
-		if (pnm->maxval == 255)
-			p += n * w * h;
-		else if (bitmap)
-			p += ((w + 7) / 8) * h;
-		else if (pnm->maxval < 255)
-			p += n * w * h;
-		else
-			p += 2 * n * w * h;
+		p += span * (size_t)pnm->height;
 	}
 	else
 	{
 		unsigned char *dp;
 		int x, y, k;
-		int w, h, n;
+		int w, h;
 
 		img = fz_new_pixmap(ctx, pnm->cs, pnm->width, pnm->height, NULL, 0);
 		dp = img->samples;
@@ -452,6 +602,8 @@ static const unsigned char *
 pam_binary_read_header(fz_context *ctx, struct info *pnm, const unsigned char *p, const unsigned char *e)
 {
 	int token = TOKEN_UNKNOWN;
+	const unsigned char *eol;
+	int seen[TOKEN_ENDHDR] = { 0 };
 
 	pnm->width = 0;
 	pnm->height = 0;
@@ -461,21 +613,38 @@ pam_binary_read_header(fz_context *ctx, struct info *pnm, const unsigned char *p
 
 	while (p < e && token != TOKEN_ENDHDR)
 	{
-		p = pnm_read_token(ctx, p, e, &token);
-		p = pnm_read_white(ctx, p, e, 0);
-		switch (token)
+		eol = pnm_read_until_eol(ctx, p, e, 0);
+
+		p = pnm_read_whites(ctx, p, eol, 0);
+
+		if (p < eol && *p != '#')
 		{
-		case TOKEN_WIDTH: p = pnm_read_number(ctx, p, e, &pnm->width); break;
-		case TOKEN_HEIGHT: p = pnm_read_number(ctx, p, e, &pnm->height); break;
-		case TOKEN_DEPTH: p = pnm_read_number(ctx, p, e, &pnm->depth); break;
-		case TOKEN_MAXVAL: p = pnm_read_number(ctx, p, e, &pnm->maxval); break;
-		case TOKEN_TUPLTYPE: p = pnm_read_tupletype(ctx, p, e, &pnm->tupletype); break;
-		case TOKEN_ENDHDR: break;
-		default: fz_throw(ctx, FZ_ERROR_GENERIC, "unknown header token in pnm image");
+			p = pnm_read_token(ctx, p, eol, &token);
+
+			if (seen[token - 1])
+				fz_throw(ctx, FZ_ERROR_FORMAT, "token occurs multiple times in pnm image");
+			seen[token - 1] = 1;
+
+			if (token != TOKEN_ENDHDR)
+			{
+				p = pnm_read_whites(ctx, p, eol, 1);
+				switch (token)
+				{
+				case TOKEN_WIDTH: pnm->width = 0; p = pnm_read_int(ctx, p, eol, &pnm->width); break;
+				case TOKEN_HEIGHT: pnm->height = 0; p = pnm_read_int(ctx, p, eol, &pnm->height); break;
+				case TOKEN_DEPTH: pnm->depth = 0; p = pnm_read_int(ctx, p, eol, &pnm->depth); break;
+				case TOKEN_MAXVAL: pnm->maxval = 0; p = pnm_read_int(ctx, p, eol, &pnm->maxval); break;
+				case TOKEN_TUPLTYPE: pnm->tupletype = 0; p = pnm_read_tupletype(ctx, p, eol, &pnm->tupletype); break;
+				}
+			}
+
+			p = pnm_read_whites(ctx, p, eol, 0);
 		}
 
-		if (token != TOKEN_ENDHDR)
-			p = pnm_read_white(ctx, p, e, 0);
+		if (p < eol && *p == '#')
+			p = pnm_read_comment(ctx, p, eol, 0);
+
+		p = pnm_read_eol(ctx, p, e, 0);
 	}
 
 	return p;
@@ -502,7 +671,7 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		case 4: pnm->tupletype = PAM_CMYK; break;
 		case 5: pnm->tupletype = PAM_CMYKA; break;
 		default:
-			fz_throw(ctx, FZ_ERROR_GENERIC, "cannot guess tuple type based on depth in pnm image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "cannot guess tuple type based on depth in pnm image");
 		}
 
 	if (pnm->tupletype == PAM_BW && pnm->maxval > 1)
@@ -544,22 +713,22 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		pnm->cs = fz_device_cmyk(ctx);
 		break;
 	default:
-		fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported tuple type");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported tuple type");
 	}
 
 	if (pnm->depth != fz_colorspace_n(ctx, pnm->cs) + pnm->alpha)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "depth out of tuple type range");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "depth out of tuple type range");
 	if (pnm->maxval < minval || pnm->maxval > maxval)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maxval out of range");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maxval out of range");
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
 
 	if (onlymeta)
 	{
@@ -583,9 +752,9 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 				packed = 1;
 		}
 		if (packed && (e < p || (size_t)(e - p) < size / 8))
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated packed image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated packed image");
 		if (!packed && (e < p || (size_t)(e - p) < size * (pnm->maxval < 256 ? 1 : 2)))
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 		if (pnm->maxval == 255)
 			p += size;
@@ -598,8 +767,7 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		else
 			p += 2 * size;
 	}
-
-	if (!onlymeta)
+	else
 	{
 		unsigned char *dp;
 		int x, y, k, packed;
@@ -627,9 +795,9 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 					packed = 1;
 			}
 			if (packed && (e < p || (size_t)(e - p) < size / 8))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "truncated packed image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "truncated packed image");
 			if (!packed && (e < p || (size_t)(e - p) < size * (pnm->maxval < 256 ? 1 : 2)))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 			if (pnm->maxval == 255)
 				memcpy(dp, p, size);
@@ -689,6 +857,108 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 	return img;
 }
 
+static const unsigned char *
+pfm_binary_read_header(fz_context *ctx, struct info *pnm, const unsigned char *p, const unsigned char *e)
+{
+	pnm->width = 0;
+	p = pnm_read_int(ctx, p, e, &pnm->width);
+	p = pnm_read_whites_and_eols(ctx, p, e,1);
+
+	pnm->height = 0;
+	p = pnm_read_int(ctx, p, e, &pnm->height);
+	p = pnm_read_whites_and_eols(ctx, p, e,1);
+
+	p = pnm_read_real(ctx, p, e, &pnm->scale);
+
+	p = pnm_read_white_or_eol(ctx, p, e);
+
+	if (pnm->scale >= 0)
+		pnm->endian = ENDIAN_BIG;
+	else
+	{
+		pnm->endian = ENDIAN_LITTLE;
+		pnm->scale = -pnm->scale;
+	}
+
+	return p;
+}
+
+static fz_pixmap *
+pfm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, const unsigned char *e, int onlymeta, int rgb, const unsigned char **out)
+{
+	fz_pixmap *pix = NULL;
+
+	fz_var(pix);
+
+	p = pfm_binary_read_header(ctx, pnm, p, e);
+	pnm->cs = rgb ? fz_device_rgb(ctx) : fz_device_gray(ctx);
+
+	if (pnm->height <= 0)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
+	if (pnm->width <= 0)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
+	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
+
+	if (onlymeta)
+	{
+		size_t w = pnm->width;
+		size_t h = pnm->height;
+		int n = fz_colorspace_n(ctx, pnm->cs);
+		size_t size = w * h * n * sizeof(float);
+
+		if (e < p || (size_t)(e - p) < size)
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
+
+		p += size;
+	}
+	else
+	{
+		float *samples = NULL;
+		float *sample;
+		int w = pnm->width;
+		int h = pnm->height;
+		int n = fz_colorspace_n(ctx, pnm->cs);
+		size_t size = (size_t) w * h * n * sizeof(float);
+		int x, y, k;
+
+		if (e < p || (size_t)(e - p) < size)
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
+
+		sample = samples = fz_malloc(ctx, size);
+		fz_try(ctx)
+		{
+			for (y = 0; y < h; y++)
+				for (x = 0; x < w; x++)
+					for (k = 0; k < n; k++)
+					{
+						uint32_t u;
+						float f;
+
+						if (pnm->endian == ENDIAN_LITTLE)
+							u = p[0] | (p[1] << 8) | (p[2] << 16) | (p[3] << 24);
+						else
+							u = p[3] | (p[2] << 8) | (p[1] << 16) | (p[0] << 24);
+						memcpy(&f, &u, sizeof(float));
+
+						*sample++ = f / pnm->scale;
+						p += sizeof(float);
+					}
+
+			pix = fz_new_pixmap_from_float_data(ctx, pnm->cs, w, h, samples);
+		}
+		fz_always(ctx)
+			fz_free(ctx, samples);
+		fz_catch(ctx)
+			fz_rethrow(ctx);
+	}
+
+	if (out)
+		*out = p;
+
+	return pix;
+}
+
 static fz_pixmap *
 pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t total, int onlymeta, int subimage)
 {
@@ -700,8 +970,9 @@ pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t
 	{
 		int subonlymeta = onlymeta || (subimage > 0);
 
+		p = pnm_read_whites_and_eols(ctx, p, e, 0);
 		p = pnm_read_signature(ctx, p, e, signature);
-		p = pnm_read_white(ctx, p, e, 0);
+		p = pnm_read_whites_and_eols(ctx, p, e, 1);
 
 		if (!strcmp(signature, "P1"))
 		{
@@ -735,8 +1006,14 @@ pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t
 		}
 		else if (!strcmp(signature, "P7"))
 			pix = pam_binary_read_image(ctx, pnm, p, e, subonlymeta, &p);
+		else if (!strcmp(signature, "Pf"))
+			pix = pfm_binary_read_image(ctx, pnm, p, e, subonlymeta, 0, &p);
+		else if (!strcmp(signature, "PF"))
+			pix = pfm_binary_read_image(ctx, pnm, p, e, subonlymeta, 1, &p);
 		else
-			fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported portable anymap signature (0x%02x, 0x%02x)", signature[0], signature[1]);
+			fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported portable anymap signature (0x%02x, 0x%02x)", signature[0], signature[1]);
+
+		p = pnm_read_whites_and_eols(ctx, p, e, 0);
 
 		if (onlymeta)
 			pnm->subimages++;
@@ -745,7 +1022,7 @@ pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t
 	}
 
 	if (p >= e && subimage >= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "subimage count out of range");
+		fz_throw(ctx, FZ_ERROR_ARGUMENT, "subimage count out of range");
 
 	return pix;
 }
