@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -99,7 +99,7 @@ typedef struct word_record_s {
 	int len;
 	fz_rect bbox;
 	int n;
-	int unicode[1];
+	int unicode[FZ_FLEXIBLE_ARRAY];
 } word_record;
 
 typedef struct fz_ocr_device_s
@@ -422,7 +422,7 @@ flush_word(fz_context *ctx, fz_ocr_device *ocr)
 			ocr->words = fz_realloc_array(ctx, ocr->words, new_max, word_record *);
 			ocr->words_max = new_max;
 		}
-		word = (word_record *)Memento_label(fz_malloc(ctx, sizeof(word_record) + sizeof(int) * (ocr->char_len-1)), "word_record");
+		word = fz_malloc_flexible(ctx, word_record, unicode, ocr->char_len);
 		word->len = ocr->char_len;
 		word->bbox = ocr->word_bbox;
 		word->n = 0;
@@ -669,7 +669,7 @@ rewrite_span(fz_context *ctx, fz_rewrite_device *dev, fz_matrix ctm, const fz_te
 
 	/* And do the actual rewriting */
 	for (i = 0; i < rspan->len; i++) {
-		float advance = fz_advance_glyph(ctx, span->font, rspan->items[i].gid, wmode);
+		float advance = rspan->items[i].adv;
 		fz_point vadv = { dir.x * advance, dir.y * advance };
 		rewrite_char(ctx, dev, ctm, &rspan->items[i], vadv);
 	}
