@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -570,7 +570,7 @@ static fz_buffer *read_zip_entry(fz_context *ctx, fz_archive *arch, const char *
 			z.next_out = ubuf->data;
 			z.avail_out = ent->usize;
 			z.next_in = cbuf;
-			z.avail_in = fz_read(ctx, file, cbuf, ent->csize);
+			z.avail_in = (uInt)fz_read(ctx, file, cbuf, ent->csize);
 			if (z.avail_in < ent->csize)
 				fz_warn(ctx, "premature end of compressed data for compressed archive entry");
 
