@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 #include "mupdf/fitz.h"
 
@@ -108,9 +108,6 @@ fz_new_pixmap_from_display_list_with_separations(fz_context *ctx, fz_display_lis
 	fz_rect rect;
 	fz_irect bbox;
 	fz_pixmap *pix;
-	fz_device *dev = NULL;
-
-	fz_var(dev);
 
 	rect = fz_bound_display_list(ctx, list);
 	rect = fz_transform_rect(rect, ctm);
@@ -122,6 +119,16 @@ fz_new_pixmap_from_display_list_with_separations(fz_context *ctx, fz_display_lis
 	else
 		fz_clear_pixmap_with_value(ctx, pix, 0xFF);
 
+	return fz_fill_pixmap_from_display_list(ctx, list, ctm, pix);
+}
+
+fz_pixmap *
+fz_fill_pixmap_from_display_list(fz_context *ctx, fz_display_list *list, fz_matrix ctm, fz_pixmap *pix)
+{
+	fz_device *dev = NULL;
+
+	fz_var(dev);
+
 	fz_try(ctx)
 	{
 		dev = fz_new_draw_device(ctx, ctm, pix);
@@ -364,6 +371,22 @@ fz_search_display_list(fz_context *ctx, fz_display_list *list, const char *needl
 	return count;
 }
 
+int
+fz_search_display_list_cb(fz_context *ctx, fz_display_list *list, const char *needle, fz_search_callback_fn *cb, void *opaque)
+{
+	fz_stext_page *text;
+	int count = 0;
+
+	text = fz_new_stext_page_from_display_list(ctx, list, NULL);
+	fz_try(ctx)
+		count = fz_search_stext_page_cb(ctx, text, needle, cb, opaque);
+	fz_always(ctx)
+		fz_drop_stext_page(ctx, text);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return count;
+}
+
 int
 fz_search_page(fz_context *ctx, fz_page *page, const char *needle, int *hit_mark, fz_quad *hit_bbox, int hit_max)
 {
@@ -381,6 +404,23 @@ fz_search_page(fz_context *ctx, fz_page *page, const char *needle, int *hit_mark
 	return count;
 }
 
+int
+fz_search_page_cb(fz_context *ctx, fz_page *page, const char *needle, fz_search_callback_fn *cb, void *opaque)
+{
+	fz_stext_options opts = { FZ_STEXT_DEHYPHENATE };
+	fz_stext_page *text;
+	int count = 0;
+
+	text = fz_new_stext_page_from_page(ctx, page, &opts);
+	fz_try(ctx)
+		count = fz_search_stext_page_cb(ctx, text, needle, cb, opaque);
+	fz_always(ctx)
+		fz_drop_stext_page(ctx, text);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return count;
+}
+
 int
 fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char *needle, int *hit_mark, fz_quad *hit_bbox, int hit_max)
 {
@@ -397,6 +437,22 @@ fz_search_page_number(fz_context *ctx, fz_document *doc, int number, const char
 	return count;
 }
 
+int
+fz_search_page_number_cb(fz_context *ctx, fz_document *doc, int number, const char *needle, fz_search_callback_fn *cb, void *opaque)
+{
+	fz_page *page;
+	int count = 0;
+
+	page = fz_load_page(ctx, doc, number);
+	fz_try(ctx)
+		count = fz_search_page_cb(ctx, page, needle, cb, opaque);
+	fz_always(ctx)
+		fz_drop_page(ctx, page);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return count;
+}
+
 int
 fz_search_chapter_page_number(fz_context *ctx, fz_document *doc, int chapter, int number, const char *needle, int *hit_mark, fz_quad *hit_bbox, int hit_max)
 {
@@ -413,6 +469,22 @@ fz_search_chapter_page_number(fz_context *ctx, fz_document *doc, int chapter, in
 	return count;
 }
 
+int
+fz_search_chapter_page_number_cb(fz_context *ctx, fz_document *doc, int chapter, int number, const char *needle, fz_search_callback_fn *cb, void *opaque)
+{
+	fz_page *page;
+	int count = 0;
+
+	page = fz_load_chapter_page(ctx, doc, chapter, number);
+	fz_try(ctx)
+		count = fz_search_page_cb(ctx, page, needle, cb, opaque);
+	fz_always(ctx)
+		fz_drop_page(ctx, page);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
+	return count;
+}
+
 fz_buffer *
 fz_new_buffer_from_stext_page(fz_context *ctx, fz_stext_page *page)
 {
@@ -745,6 +817,7 @@ fz_append_image_as_data_uri(fz_context *ctx, fz_buffer *out, fz_image *image)
 {
 	fz_compressed_buffer *cbuf;
 	fz_buffer *buf;
+	const char *mime;
 
 	cbuf = fz_compressed_image_buffer(ctx, image);
 
@@ -764,6 +837,7 @@ fz_append_image_as_data_uri(fz_context *ctx, fz_buffer *out, fz_image *image)
 			return;
 		}
 	}
+
 	if (cbuf && cbuf->params.type == FZ_IMAGE_PNG)
 	{
 		fz_append_string(ctx, out, "data:image/png;base64,");
@@ -771,10 +845,21 @@ fz_append_image_as_data_uri(fz_context *ctx, fz_buffer *out, fz_image *image)
 		return;
 	}
 
-	buf = fz_new_buffer_from_image_as_png(ctx, image, fz_default_color_params);
+	if (fz_is_lossy_image(ctx, image))
+	{
+		/* Convert lossy image formats to JPEG */
+		buf = fz_new_buffer_from_image_as_jpeg(ctx, image, fz_default_color_params, 90, 0);
+		mime = "data:image/jpeg;base64,";
+	}
+	else
+	{
+		buf = fz_new_buffer_from_image_as_png(ctx, image, fz_default_color_params);
+		mime = "data:image/png;base64,";
+	}
+
 	fz_try(ctx)
 	{
-		fz_append_string(ctx, out, "data:image/png;base64,");
+		fz_append_string(ctx, out, mime);
 		fz_append_base64_buffer(ctx, out, buf, 1);
 	}
 	fz_always(ctx)
