@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -23,6 +23,8 @@
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
+#include "cmaps/TrueType-UCS2.h"
+
 #include <string.h>
 
 static pdf_cmap *
@@ -53,16 +55,39 @@ pdf_load_embedded_cmap_imp(fz_context *ctx, pdf_document *doc, pdf_obj *stmobj,
 		obj = pdf_dict_get(ctx, stmobj, PDF_NAME(UseCMap));
 		if (pdf_is_name(ctx, obj))
 		{
-			usecmap = pdf_load_system_cmap(ctx, pdf_to_name(ctx, obj));
-			pdf_set_usecmap(ctx, cmap, usecmap);
+			fz_try(ctx)
+			{
+				usecmap = pdf_load_system_cmap(ctx, pdf_to_name(ctx, obj));
+				pdf_set_usecmap(ctx, cmap, usecmap);
+			}
+			fz_catch(ctx)
+			{
+				fz_rethrow_if(ctx, FZ_ERROR_SYSTEM);
+				fz_report_error(ctx);
+				fz_warn(ctx, "cannot load system CMap: %s", pdf_to_name(ctx, obj));
+			}
 		}
 		else if (pdf_is_indirect(ctx, obj))
 		{
 			if (pdf_cycle(ctx, &cycle, cycle_up, obj))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "recursive CMap");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "recursive CMap");
 			usecmap = pdf_load_embedded_cmap_imp(ctx, doc, obj, &cycle);
 			pdf_set_usecmap(ctx, cmap, usecmap);
 		}
+		else if (strlen(cmap->usecmap_name) > 0)
+		{
+			fz_try(ctx)
+			{
+				usecmap = pdf_load_system_cmap(ctx, cmap->usecmap_name);
+				pdf_set_usecmap(ctx, cmap, usecmap);
+			}
+			fz_catch(ctx)
+			{
+				fz_rethrow_if(ctx, FZ_ERROR_SYSTEM);
+				fz_report_error(ctx);
+				fz_warn(ctx, "cannot load system CMap: %s", pdf_to_name(ctx, obj));
+			}
+		}
 
 		pdf_store_item(ctx, stmobj, cmap, pdf_cmap_size(ctx, cmap));
 	}
@@ -117,6 +142,7 @@ pdf_load_builtin_cmap(fz_context *ctx, const char *name)
 {
 	if (!strcmp(name, "Identity-H")) return pdf_new_identity_cmap(ctx, 0, 2);
 	if (!strcmp(name, "Identity-V")) return pdf_new_identity_cmap(ctx, 1, 2);
+	if (!strcmp(name, "TrueType-UCS2")) return &cmap_TrueType_UCS2;
 	return NULL;
 }
 
@@ -243,6 +269,7 @@ static pdf_cmap *table[] = {
 	&cmap_KSCms_UHC_HW_V,
 	&cmap_KSCms_UHC_V,
 	&cmap_KSCpc_EUC_H,
+	&cmap_TrueType_UCS2,
 	&cmap_UniCNS_UCS2_H,
 	&cmap_UniCNS_UCS2_V,
 	&cmap_UniCNS_UTF16_H,
@@ -297,13 +324,13 @@ pdf_load_system_cmap(fz_context *ctx, const char *cmap_name)
 
 	cmap = pdf_load_builtin_cmap(ctx, cmap_name);
 	if (!cmap)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "no builtin cmap file: %s", cmap_name);
+		fz_throw(ctx, FZ_ERROR_FORMAT, "no builtin cmap file: %s", cmap_name);
 
 	if (cmap->usecmap_name[0] && !cmap->usecmap)
 	{
 		usecmap = pdf_load_system_cmap(ctx, cmap->usecmap_name);
 		if (!usecmap)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "no builtin cmap file: %s", cmap->usecmap_name);
+			fz_throw(ctx, FZ_ERROR_FORMAT, "no builtin cmap file: %s", cmap->usecmap_name);
 		pdf_set_usecmap(ctx, cmap, usecmap);
 	}
 
