@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
@@ -36,13 +36,16 @@
 #define FT_SFNT_HEAD ft_sfnt_head
 #endif
 
-static int ft_font_file_kind(FT_Face face)
+static int ft_font_file_kind(fz_context *ctx, FT_Face face)
 {
+	const char *kind;
+	fz_ft_lock(ctx);
 #ifdef FT_FONT_FORMATS_H
-	const char *kind = FT_Get_Font_Format(face);
+	kind = FT_Get_Font_Format(face);
 #else
-	const char *kind = FT_Get_X11_Font_Format(face);
+	kind = FT_Get_X11_Font_Format(face);
 #endif
+	fz_ft_unlock(ctx);
 	if (!strcmp(kind, "TrueType")) return 2;
 	if (!strcmp(kind, "Type 1")) return 1;
 	if (!strcmp(kind, "CFF")) return 3;
@@ -57,14 +60,14 @@ static int is_ttc(fz_font *font)
 	return !memcmp(font->buffer->data, "ttcf", 4);
 }
 
-static int is_truetype(FT_Face face)
+static int is_truetype(fz_context *ctx, FT_Face face)
 {
-	return ft_font_file_kind(face) == 2;
+	return ft_font_file_kind(ctx, face) == 2;
 }
 
-static int is_postscript(FT_Face face)
+static int is_postscript(fz_context *ctx, FT_Face face)
 {
-	int kind = ft_font_file_kind(face);
+	int kind = ft_font_file_kind(ctx, face);
 	return (kind == 1 || kind == 3);
 }
 
@@ -103,9 +106,10 @@ pdf_add_font_file(fz_context *ctx, pdf_document *doc, fz_font *font)
 	fz_try(ctx)
 	{
 		size_t len = fz_buffer_storage(ctx, buf, NULL);
+		int is_opentype;
 		obj = pdf_new_dict(ctx, doc, 3);
 		pdf_dict_put_int(ctx, obj, PDF_NAME(Length1), (int)len);
-		switch (ft_font_file_kind(font->ft_face))
+		switch (ft_font_file_kind(ctx, font->ft_face))
 		{
 		case 1:
 			/* TODO: these may not be the correct values, but I doubt it matters */
@@ -115,7 +119,10 @@ pdf_add_font_file(fz_context *ctx, pdf_document *doc, fz_font *font)
 		case 2:
 			break;
 		case 3:
-			if (FT_Get_Sfnt_Table(font->ft_face, FT_SFNT_HEAD))
+			fz_ft_lock(ctx);
+			is_opentype = !!FT_Get_Sfnt_Table(font->ft_face, FT_SFNT_HEAD);
+			fz_ft_unlock(ctx);
+			if (is_opentype)
 				pdf_dict_put(ctx, obj, PDF_NAME(Subtype), PDF_NAME(OpenType));
 			else
 				pdf_dict_put(ctx, obj, PDF_NAME(Subtype), PDF_NAME(CIDFontType0C));
@@ -167,7 +174,7 @@ pdf_add_font_descriptor(fz_context *ctx, pdf_document *doc, pdf_obj *fobj, fz_fo
 		fileref = pdf_add_font_file(ctx, doc, font);
 		if (fileref)
 		{
-			switch (ft_font_file_kind(face))
+			switch (ft_font_file_kind(ctx, face))
 			{
 			default:
 			case 1: pdf_dict_put_drop(ctx, fdobj, PDF_NAME(FontFile), fileref); break;
@@ -374,14 +381,16 @@ pdf_add_descendant_cid_font(fz_context *ctx, pdf_document *doc, fz_font *font)
 	fz_try(ctx)
 	{
 		pdf_dict_put(ctx, fobj, PDF_NAME(Type), PDF_NAME(Font));
-		if (is_truetype(face))
+		if (is_truetype(ctx, face))
 			pdf_dict_put(ctx, fobj, PDF_NAME(Subtype), PDF_NAME(CIDFontType2));
 		else
 			pdf_dict_put(ctx, fobj, PDF_NAME(Subtype), PDF_NAME(CIDFontType0));
 
 		pdf_add_cid_system_info(ctx, doc, fobj, "Adobe", "Identity", 0);
 
+		fz_ft_lock(ctx);
 		ps_name = FT_Get_Postscript_Name(face);
+		fz_ft_unlock(ctx);
 		if (ps_name)
 			pdf_dict_put_name(ctx, fobj, PDF_NAME(BaseFont), ps_name);
 		else
@@ -419,7 +428,7 @@ static void
 pdf_add_to_unicode(fz_context *ctx, pdf_document *doc, pdf_obj *fobj, fz_font *font)
 {
 	FT_Face face = font->ft_face;
-	fz_buffer *buf;
+	fz_buffer *buf = NULL;
 
 	int *table;
 	int num_seq = 0;
@@ -432,7 +441,7 @@ pdf_add_to_unicode(fz_context *ctx, pdf_document *doc, pdf_obj *fobj, fz_font *f
 		FT_UInt gid;
 
 		table = fz_calloc(ctx, face->num_glyphs, sizeof *table);
-		fz_lock(ctx, FZ_LOCK_FREETYPE);
+		fz_ft_lock(ctx);
 		ucs = FT_Get_First_Char(face, &gid);
 		while (gid > 0)
 		{
@@ -440,7 +449,7 @@ pdf_add_to_unicode(fz_context *ctx, pdf_document *doc, pdf_obj *fobj, fz_font *f
 				table[gid] = ucs;
 			ucs = FT_Get_Next_Char(face, ucs, &gid);
 		}
-		fz_unlock(ctx, FZ_LOCK_FREETYPE);
+		fz_ft_unlock(ctx);
 	}
 
 	for (k = 0; k < face->num_glyphs; k += n)
@@ -460,9 +469,12 @@ pdf_add_to_unicode(fz_context *ctx, pdf_document *doc, pdf_obj *fobj, fz_font *f
 		return;
 	}
 
-	buf = fz_new_buffer(ctx, 0);
+	fz_var(buf);
+
 	fz_try(ctx)
 	{
+		buf = fz_new_buffer(ctx, 0);
+
 		/* Header boiler plate */
 		fz_append_string(ctx, buf, "/CIDInit /ProcSet findresource begin\n");
 		fz_append_string(ctx, buf, "12 dict begin\n");
@@ -667,14 +679,17 @@ pdf_add_simple_font(fz_context *ctx, pdf_document *doc, fz_font *font, int encod
 	fz_try(ctx)
 	{
 		pdf_dict_put(ctx, fobj, PDF_NAME(Type), PDF_NAME(Font));
-		if (is_truetype(face))
+		if (is_truetype(ctx, face))
 			pdf_dict_put(ctx, fobj, PDF_NAME(Subtype), PDF_NAME(TrueType));
 		else
 			pdf_dict_put(ctx, fobj, PDF_NAME(Subtype), PDF_NAME(Type1));
 
 		if (!is_builtin_font(ctx, font))
 		{
-			const char *ps_name = FT_Get_Postscript_Name(face);
+			const char *ps_name;
+			fz_ft_lock(ctx);
+			ps_name = FT_Get_Postscript_Name(face);
+			fz_ft_unlock(ctx);
 			if (!ps_name)
 				ps_name = font->name;
 			pdf_dict_put_name(ctx, fobj, PDF_NAME(BaseFont), ps_name);
@@ -702,15 +717,15 @@ pdf_add_simple_font(fz_context *ctx, pdf_document *doc, fz_font *font, int encod
 }
 
 int
-pdf_font_writing_supported(fz_font *font)
+pdf_font_writing_supported(fz_context *ctx, fz_font *font)
 {
 	if (font->ft_face == NULL || font->buffer == NULL || font->buffer->len < 4 || !font->flags.embed || font->flags.never_embed)
 		return 0;
 	if (is_ttc(font))
 		return 1;
-	if (is_truetype(font->ft_face))
+	if (is_truetype(ctx, font->ft_face))
 		return 1;
-	if (is_postscript(font->ft_face))
+	if (is_postscript(ctx, font->ft_face))
 		return 1;
 	return 0;
 }
@@ -807,6 +822,5 @@ pdf_add_cjk_font(fz_context *ctx, pdf_document *doc, fz_font *fzfont, int script
 pdf_obj *
 pdf_add_substitute_font(fz_context *ctx, pdf_document *doc, fz_font *font)
 {
-	fz_throw(ctx, FZ_ERROR_GENERIC, "substitute font creation is not implemented yet");
-	return NULL;
+	fz_throw(ctx, FZ_ERROR_UNSUPPORTED, "substitute font creation is not implemented yet");
 }
