@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2024 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -23,6 +23,8 @@
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
+#include "pdf-imp.h"
+
 #include <string.h>
 
 typedef enum
@@ -31,6 +33,7 @@ typedef enum
 	FLUSH_COLOR_F = 2,
 	FLUSH_COLOR_S = 4,
 	FLUSH_TEXT = 8,
+	FLUSH_OP = 16,
 
 	FLUSH_ALL = 15,
 	FLUSH_STROKE = 1+4,
@@ -227,29 +230,6 @@ filter_pop(fz_context *ctx, pdf_sanitize_processor *p)
 	return 0;
 }
 
-/* We never allow the topmost gstate to be changed. This allows us
- * to pop back to the zeroth level and be sure that our gstate is
- * sane. This is important for being able to add new operators at
- * the end of pages in a sane way. */
-static filter_gstate *
-gstate_to_update(fz_context *ctx, pdf_sanitize_processor *p)
-{
-	filter_gstate *gstate = p->gstate;
-
-	/* If we're not the top, that's fine */
-	if (gstate->next != NULL)
-		return gstate;
-
-	/* We are the top. Push a group, so we're not */
-	filter_push(ctx, p);
-	gstate = p->gstate;
-	gstate->pushed = 1;
-	if (p->chain->op_q)
-		p->chain->op_q(ctx, p->chain);
-
-	return p->gstate;
-}
-
 static void flush_tags(fz_context *ctx, pdf_sanitize_processor *p, tag_record **tags)
 {
 	tag_record *tag = *tags;
@@ -270,25 +250,56 @@ static void flush_tags(fz_context *ctx, pdf_sanitize_processor *p, tag_record **
 	*tags = NULL;
 }
 
+static filter_gstate *
+ensure_pushed(fz_context *ctx, pdf_sanitize_processor *p)
+{
+	filter_gstate *gstate = p->gstate;
+
+	if (gstate->next)
+	{
+		/* We are not the top gstate. */
+		if (p->gstate->pushed == 0)
+		{
+			p->gstate->pushed = 1;
+			if (p->chain->op_q)
+				p->chain->op_q(ctx, p->chain);
+		}
+
+		return gstate;
+	}
+
+	/* So gstate is the top one. We want at least one pushed. */
+	filter_push(ctx, p);
+	p->gstate->pushed = 1;
+	if (p->chain->op_q)
+		p->chain->op_q(ctx, p->chain);
+
+	/* Now, gstate->pending has all been copied onto new_gstate->pending.
+	 * So put, gstate->pending back to sanity. */
+	pdf_drop_font(ctx, gstate->pending.text.font);
+	fz_drop_string(ctx, gstate->pending.text.fontname);
+	gstate->pending = p->gstate->next->sent;
+	pdf_keep_font(ctx, gstate->pending.text.font);
+	fz_keep_string(ctx, gstate->pending.text.fontname);
+
+	return p->gstate;
+}
+
 static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 {
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	int i;
 
 	/* No point in sending anything if we're clipping it away! */
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	if (gstate->pushed == 0)
-	{
-		gstate->pushed = 1;
-		if (p->chain->op_q)
-			p->chain->op_q(ctx, p->chain);
-	}
-
 	if (flush)
 		flush_tags(ctx, p, &p->pending_tags);
 
+	if (flush & FLUSH_OP)
+		gstate = ensure_pushed(ctx, p);
+
 	if (flush & FLUSH_CTM)
 	{
 		if (gstate->pending.ctm.a != 1 || gstate->pending.ctm.b != 0 ||
@@ -297,6 +308,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 		{
 			fz_matrix current = gstate->sent.ctm;
 
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_cm)
 				p->chain->op_cm(ctx, p->chain,
 					gstate->pending.ctm.a,
@@ -321,6 +333,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 		if (gstate->pending.cs.cs == fz_device_gray(ctx) && !gstate->pending.sc.pat && !gstate->pending.sc.shd && gstate->pending.sc.n == 1 &&
 			(gstate->sent.cs.cs != fz_device_gray(ctx) || gstate->sent.sc.pat || gstate->sent.sc.shd || gstate->sent.sc.n != 1 || gstate->pending.sc.c[0] != gstate->sent.sc.c[0]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_g)
 				p->chain->op_g(ctx, p->chain, gstate->pending.sc.c[0]);
 			goto done_sc;
@@ -329,6 +342,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 			(gstate->sent.cs.cs != fz_device_rgb(ctx) || gstate->sent.sc.pat || gstate->sent.sc.shd || gstate->sent.sc.n != 3 || gstate->pending.sc.c[0] != gstate->sent.sc.c[0] ||
 				gstate->pending.sc.c[1] != gstate->sent.sc.c[1] || gstate->pending.sc.c[1] != gstate->sent.sc.c[1]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_rg)
 				p->chain->op_rg(ctx, p->chain, gstate->pending.sc.c[0], gstate->pending.sc.c[1], gstate->pending.sc.c[2]);
 			goto done_sc;
@@ -337,6 +351,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 			(gstate->sent.cs.cs != fz_device_cmyk(ctx) || gstate->sent.sc.pat || gstate->sent.sc.shd || gstate->pending.sc.n != 4 || gstate->pending.sc.c[0] != gstate->sent.sc.c[0] ||
 				gstate->pending.sc.c[1] != gstate->sent.sc.c[1] || gstate->pending.sc.c[2] != gstate->sent.sc.c[2] || gstate->pending.sc.c[3] != gstate->sent.sc.c[3]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_k)
 				p->chain->op_k(ctx, p->chain, gstate->pending.sc.c[0], gstate->pending.sc.c[1], gstate->pending.sc.c[2], gstate->pending.sc.c[3]);
 			goto done_sc;
@@ -344,6 +359,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 
 		if (strcmp(gstate->pending.cs.name, gstate->sent.cs.name))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_cs)
 				p->chain->op_cs(ctx, p->chain, gstate->pending.cs.name, gstate->pending.cs.cs);
 		}
@@ -362,6 +378,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 						emit = 1;
 			if (emit)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (gstate->pending.sc.pat)
 					if (p->chain->op_sc_pattern)
 						p->chain->op_sc_pattern(ctx, p->chain, gstate->pending.sc.name, gstate->pending.sc.pat, gstate->pending.sc.n, gstate->pending.sc.c);
@@ -383,6 +400,7 @@ static void filter_flush(fz_context *ctx, pdf_sanitize_processor *p, int flush)
 						emit = 1;
 			if (emit)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_sc_color)
 					p->chain->op_sc_color(ctx, p->chain, gstate->pending.sc.n, gstate->pending.sc.c);
 			}
@@ -398,6 +416,7 @@ done_sc:
 		if (gstate->pending.CS.cs == fz_device_gray(ctx) && !gstate->pending.SC.pat && !gstate->pending.SC.shd && gstate->pending.SC.n == 1 &&
 			(gstate->sent.CS.cs != fz_device_gray(ctx) || gstate->sent.SC.pat || gstate->sent.SC.shd || gstate->sent.SC.n != 0 || gstate->pending.SC.c[0] != gstate->sent.SC.c[0]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_G)
 				p->chain->op_G(ctx, p->chain, gstate->pending.SC.c[0]);
 			goto done_SC;
@@ -406,6 +425,7 @@ done_sc:
 			(gstate->sent.CS.cs != fz_device_rgb(ctx) || gstate->sent.SC.pat || gstate->sent.SC.shd || gstate->sent.SC.n != 3 || gstate->pending.SC.c[0] != gstate->sent.SC.c[0] ||
 				gstate->pending.SC.c[1] != gstate->sent.SC.c[1] || gstate->pending.SC.c[1] != gstate->sent.SC.c[1]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_RG)
 				p->chain->op_RG(ctx, p->chain, gstate->pending.SC.c[0], gstate->pending.SC.c[1], gstate->pending.SC.c[2]);
 			goto done_SC;
@@ -414,6 +434,7 @@ done_sc:
 			(gstate->sent.CS.cs != fz_device_cmyk(ctx) || gstate->sent.SC.pat || gstate->sent.SC.shd || gstate->pending.SC.n != 4 || gstate->pending.SC.c[0] != gstate->sent.SC.c[0] ||
 				gstate->pending.SC.c[1] != gstate->sent.SC.c[1] || gstate->pending.SC.c[2] != gstate->sent.SC.c[2] || gstate->pending.SC.c[3] != gstate->sent.SC.c[3]))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_K)
 				p->chain->op_K(ctx, p->chain, gstate->pending.SC.c[0], gstate->pending.SC.c[1], gstate->pending.SC.c[2], gstate->pending.SC.c[3]);
 			goto done_SC;
@@ -421,6 +442,7 @@ done_sc:
 
 		if (strcmp(gstate->pending.CS.name, gstate->sent.CS.name))
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_CS)
 				p->chain->op_CS(ctx, p->chain, gstate->pending.CS.name, gstate->pending.CS.cs);
 		}
@@ -439,6 +461,7 @@ done_sc:
 						emit = 1;
 			if (emit)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (gstate->pending.SC.pat)
 					if (p->chain->op_SC_pattern)
 						p->chain->op_SC_pattern(ctx, p->chain, gstate->pending.SC.name, gstate->pending.SC.pat, gstate->pending.SC.n, gstate->pending.SC.c);
@@ -460,6 +483,7 @@ done_sc:
 						emit = 1;
 			if (emit)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_SC_color)
 					p->chain->op_SC_color(ctx, p->chain, gstate->pending.SC.n, gstate->pending.SC.c);
 			}
@@ -474,21 +498,25 @@ done_SC:
 	{
 		if (gstate->pending.stroke.linecap != gstate->sent.stroke.linecap)
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_J)
 				p->chain->op_J(ctx, p->chain, gstate->pending.stroke.linecap);
 		}
 		if (gstate->pending.stroke.linejoin != gstate->sent.stroke.linejoin)
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_j)
 				p->chain->op_j(ctx, p->chain, gstate->pending.stroke.linejoin);
 		}
 		if (gstate->pending.stroke.linewidth != gstate->sent.stroke.linewidth)
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_w)
 				p->chain->op_w(ctx, p->chain, gstate->pending.stroke.linewidth);
 		}
 		if (gstate->pending.stroke.miterlimit != gstate->sent.stroke.miterlimit)
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_M)
 				p->chain->op_M(ctx, p->chain, gstate->pending.stroke.miterlimit);
 		}
@@ -499,6 +527,7 @@ done_SC:
 	{
 		if (p->BT_pending)
 		{
+			gstate = ensure_pushed(ctx, p);
 			if (p->chain->op_BT)
 				p->chain->op_BT(ctx, p->chain);
 			p->BT_pending = 0;
@@ -510,22 +539,26 @@ done_SC:
 		{
 			if (gstate->pending.text.char_space != gstate->sent.text.char_space)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tc)
 					p->chain->op_Tc(ctx, p->chain, gstate->pending.text.char_space);
 			}
 			if (gstate->pending.text.word_space != gstate->sent.text.word_space)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tw)
 					p->chain->op_Tw(ctx, p->chain, gstate->pending.text.word_space);
 			}
 			if (gstate->pending.text.scale != gstate->sent.text.scale)
 			{
 				/* The value of scale in the gstate is divided by 100 from what is written in the file */
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tz)
 					p->chain->op_Tz(ctx, p->chain, gstate->pending.text.scale*100);
 			}
 			if (gstate->pending.text.leading != gstate->sent.text.leading)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_TL)
 					p->chain->op_TL(ctx, p->chain, gstate->pending.text.leading);
 			}
@@ -533,16 +566,19 @@ done_SC:
 				gstate->pending.text.size != gstate->sent.text.size ||
 				gstate->pending.text.fontname != gstate->sent.text.fontname)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tf)
 					p->chain->op_Tf(ctx, p->chain, fz_cstring_from_string(gstate->pending.text.fontname), gstate->pending.text.font, gstate->pending.text.size);
 			}
 			if (gstate->pending.text.render != gstate->sent.text.render)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tr)
 					p->chain->op_Tr(ctx, p->chain, gstate->pending.text.render);
 			}
 			if (gstate->pending.text.rise != gstate->sent.text.rise)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Ts)
 					p->chain->op_Ts(ctx, p->chain, gstate->pending.text.rise);
 			}
@@ -554,6 +590,7 @@ done_SC:
 
 			if (p->Td_pending != 0)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Td)
 					p->chain->op_Td(ctx, p->chain, p->Td_value.x, p->Td_value.y);
 				p->Tm_pending = 0;
@@ -561,6 +598,7 @@ done_SC:
 			}
 			else if (p->Tm_pending != 0)
 			{
+				gstate = ensure_pushed(ctx, p);
 				if (p->chain->op_Tm)
 					p->chain->op_Tm(ctx, p->chain, p->tos.tlm.a, p->tos.tlm.b, p->tos.tlm.c, p->tos.tlm.d, p->tos.tlm.e, p->tos.tlm.f);
 				p->Tm_pending = 0;
@@ -1013,7 +1051,7 @@ static void
 pdf_filter_w(fz_context *ctx, pdf_processor *proc, float linewidth)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -1025,7 +1063,7 @@ static void
 pdf_filter_j(fz_context *ctx, pdf_processor *proc, int linejoin)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -1037,7 +1075,7 @@ static void
 pdf_filter_J(fz_context *ctx, pdf_processor *proc, int linecap)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -1049,7 +1087,7 @@ static void
 pdf_filter_M(fz_context *ctx, pdf_processor *proc, float miterlimit)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -1065,7 +1103,7 @@ pdf_filter_d(fz_context *ctx, pdf_processor *proc, pdf_obj *array, float phase)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_d)
 		p->chain->op_d(ctx, p->chain, array, phase);
 }
@@ -1078,7 +1116,7 @@ pdf_filter_ri(fz_context *ctx, pdf_processor *proc, const char *intent)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_ri)
 		p->chain->op_ri(ctx, p->chain, intent);
 }
@@ -1091,7 +1129,7 @@ pdf_filter_gs_OP(fz_context *ctx, pdf_processor *proc, int b)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_gs_OP)
 		p->chain->op_gs_OP(ctx, p->chain, b);
 }
@@ -1104,7 +1142,7 @@ pdf_filter_gs_op(fz_context *ctx, pdf_processor *proc, int b)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_gs_op)
 		p->chain->op_gs_op(ctx, p->chain, b);
 }
@@ -1117,7 +1155,7 @@ pdf_filter_gs_OPM(fz_context *ctx, pdf_processor *proc, int i)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_gs_OPM)
 		p->chain->op_gs_OPM(ctx, p->chain, i);
 }
@@ -1130,7 +1168,7 @@ pdf_filter_gs_UseBlackPtComp(fz_context *ctx, pdf_processor *proc, pdf_obj *name
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_gs_UseBlackPtComp)
 		p->chain->op_gs_UseBlackPtComp(ctx, p->chain, name);
 }
@@ -1143,7 +1181,7 @@ pdf_filter_i(fz_context *ctx, pdf_processor *proc, float flatness)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_i)
 		p->chain->op_i(ctx, p->chain, flatness);
 }
@@ -1156,7 +1194,7 @@ pdf_filter_gs_begin(fz_context *ctx, pdf_processor *proc, const char *name, pdf_
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, FLUSH_ALL);
+	filter_flush(ctx, p, FLUSH_ALL | FLUSH_OP);
 	if (p->chain->op_gs_begin)
 		p->chain->op_gs_begin(ctx, p->chain, name, extgstate);
 	copy_resource(ctx, p, PDF_NAME(ExtGState), name);
@@ -1236,7 +1274,7 @@ static void
 pdf_filter_cm(fz_context *ctx, pdf_processor *proc, float a, float b, float c, float d, float e, float f)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	fz_matrix ctm;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
@@ -1402,7 +1440,7 @@ cull_replay_rectto(fz_context *ctx, void *arg, float x1, float y1, float x2, flo
 typedef struct
 {
 	pdf_sanitize_processor *p;
-	fz_stroke_state sstate;
+	fz_stroke_state *sstate;
 	fz_path *segment;
 	fz_matrix ctm;
 	int any_sent;
@@ -1430,7 +1468,7 @@ end_segment(fz_context *ctx, segmenter_data_t *sd)
 	if (sd->segment == NULL)
 		return;
 
-	st = (sd->type == FZ_CULL_PATH_STROKE || sd->type == FZ_CULL_PATH_FILL_STROKE) ? &sd->sstate : NULL;
+	st = (sd->type == FZ_CULL_PATH_STROKE || sd->type == FZ_CULL_PATH_FILL_STROKE) ? sd->sstate : NULL;
 	r = fz_bound_path(ctx, sd->segment, st, sd->ctm);
 
 	if (sd->p->options->culler && sd->p->options->culler(ctx, sd->p->options->opaque, r, sd->type))
@@ -1530,7 +1568,7 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type, int flush)
 
 	if (p->options->culler == NULL)
 	{
-		filter_flush(ctx, p, flush);
+		filter_flush(ctx, p, flush | FLUSH_OP);
 		fz_walk_path(ctx, p->path, &cull_replay_walker, p);
 
 		if (p->gstate->clip_op != NO_CLIP_OP)
@@ -1564,16 +1602,14 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type, int flush)
 
 	if (type == FZ_CULL_PATH_STROKE || type == FZ_CULL_PATH_FILL_STROKE)
 	{
-		sd.sstate.refs = -1;
-		sd.sstate.start_cap = p->gstate->pending.stroke.linecap;
-		sd.sstate.dash_cap = p->gstate->pending.stroke.linecap;
-		sd.sstate.end_cap = p->gstate->pending.stroke.linecap;
-		sd.sstate.linejoin = p->gstate->pending.stroke.linejoin;
-		sd.sstate.linewidth = p->gstate->pending.stroke.linewidth;
-		sd.sstate.miterlimit = p->gstate->pending.stroke.miterlimit;
+		sd.sstate = fz_new_stroke_state(ctx);
+		sd.sstate->start_cap = p->gstate->pending.stroke.linecap;
+		sd.sstate->dash_cap = p->gstate->pending.stroke.linecap;
+		sd.sstate->end_cap = p->gstate->pending.stroke.linecap;
+		sd.sstate->linejoin = p->gstate->pending.stroke.linejoin;
+		sd.sstate->linewidth = p->gstate->pending.stroke.linewidth;
+		sd.sstate->miterlimit = p->gstate->pending.stroke.miterlimit;
 		/* Ignore dash for now. */
-		sd.sstate.dash_phase = 0;
-		sd.sstate.dash_len = 0;
 	}
 
 	sd.p = p;
@@ -1581,14 +1617,17 @@ cull_path(fz_context *ctx, pdf_sanitize_processor *p, int type, int flush)
 	sd.type = type;
 	if (p->gstate->clip_op != NO_CLIP_OP)
 		sd.type += FZ_CULL_CLIP_PATH_FILL - FZ_CULL_PATH_FILL;
-	sd.flush = flush;
+	sd.flush = flush | FLUSH_OP;
 	fz_try(ctx)
 	{
 		fz_walk_path(ctx, p->path, &segmenter, &sd);
 		end_segment(ctx, &sd);
 	}
 	fz_always(ctx)
+	{
 		fz_drop_path(ctx, sd.segment);
+		fz_drop_stroke_state(ctx, sd.sstate);
+	}
 	fz_catch(ctx)
 		fz_rethrow(ctx);
 
@@ -1828,7 +1867,7 @@ pdf_filter_ET(fz_context *ctx, pdf_processor *proc)
 
 	if (!p->BT_pending)
 	{
-		filter_flush(ctx, p, 0);
+		filter_flush(ctx, p, FLUSH_OP);
 		if (p->chain->op_ET)
 			p->chain->op_ET(ctx, p->chain);
 		p->in_BT = 0;
@@ -1878,7 +1917,7 @@ pdf_filter_Tc(fz_context *ctx, pdf_processor *proc, float charspace)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.char_space = charspace;
 }
 
@@ -1890,7 +1929,7 @@ pdf_filter_Tw(fz_context *ctx, pdf_processor *proc, float wordspace)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.word_space = wordspace;
 }
 
@@ -1904,7 +1943,7 @@ pdf_filter_Tz(fz_context *ctx, pdf_processor *proc, float scale)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.scale = scale / 100;
 }
 
@@ -1916,7 +1955,7 @@ pdf_filter_TL(fz_context *ctx, pdf_processor *proc, float leading)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.leading = leading;
 }
 
@@ -1928,7 +1967,7 @@ pdf_filter_Tf(fz_context *ctx, pdf_processor *proc, const char *name, pdf_font_d
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	fz_drop_string(ctx, p->gstate->pending.text.fontname);
 	p->gstate->pending.text.fontname = NULL;
 	p->gstate->pending.text.fontname = name ? fz_new_string(ctx, name) : NULL;
@@ -1946,7 +1985,7 @@ pdf_filter_Tr(fz_context *ctx, pdf_processor *proc, int render)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.render = render;
 }
 
@@ -1958,7 +1997,7 @@ pdf_filter_Ts(fz_context *ctx, pdf_processor *proc, float rise)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	p->gstate->pending.text.rise = rise;
 }
 
@@ -2108,7 +2147,7 @@ pdf_filter_d0(fz_context *ctx, pdf_processor *proc, float wx, float wy)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_d0)
 		p->chain->op_d0(ctx, p->chain, wx, wy);
 }
@@ -2121,7 +2160,7 @@ pdf_filter_d1(fz_context *ctx, pdf_processor *proc, float wx, float wy, float ll
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_d1)
 		p->chain->op_d1(ctx, p->chain, wx, wy, llx, lly, urx, ury);
 }
@@ -2172,7 +2211,7 @@ static void
 pdf_filter_CS(fz_context *ctx, pdf_processor *proc, const char *name, fz_colorspace *cs)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -2187,7 +2226,7 @@ static void
 pdf_filter_cs(fz_context *ctx, pdf_processor *proc, const char *name, fz_colorspace *cs)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -2202,7 +2241,7 @@ static void
 pdf_filter_SC_pattern(fz_context *ctx, pdf_processor *proc, const char *name, pdf_pattern *pat, int n, float *color)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	int i;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
@@ -2221,7 +2260,7 @@ static void
 pdf_filter_sc_pattern(fz_context *ctx, pdf_processor *proc, const char *name, pdf_pattern *pat, int n, float *color)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	int i;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
@@ -2240,7 +2279,7 @@ static void
 pdf_filter_SC_shade(fz_context *ctx, pdf_processor *proc, const char *name, fz_shade *shade)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -2256,7 +2295,7 @@ static void
 pdf_filter_sc_shade(fz_context *ctx, pdf_processor *proc, const char *name, fz_shade *shade)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
@@ -2272,7 +2311,7 @@ static void
 pdf_filter_SC_color(fz_context *ctx, pdf_processor *proc, int n, float *color)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	int i;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
@@ -2290,7 +2329,7 @@ static void
 pdf_filter_sc_color(fz_context *ctx, pdf_processor *proc, int n, float *color)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
-	filter_gstate *gstate = gstate_to_update(ctx, p);
+	filter_gstate *gstate = p->gstate;
 	int i;
 
 	if (fz_is_empty_rect(p->gstate->clip_rect))
@@ -2578,7 +2617,7 @@ pdf_filter_MP(fz_context *ctx, pdf_processor *proc, const char *tag)
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_MP)
 		p->chain->op_MP(ctx, p->chain, tag);
 }
@@ -2591,7 +2630,7 @@ pdf_filter_DP(fz_context *ctx, pdf_processor *proc, const char *tag, pdf_obj *ra
 	if (fz_is_empty_rect(p->gstate->clip_rect))
 		return;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_DP)
 		p->chain->op_DP(ctx, p->chain, tag, raw, cooked);
 }
@@ -2650,7 +2689,7 @@ pdf_filter_BDC(fz_context *ctx, pdf_processor *proc, const char *tag, pdf_obj *r
 	if (!pdf_is_number(ctx, mcid))
 		return;
 	bdc->mcid_num = pdf_to_int(ctx, mcid);
-	bdc->mcid_obj = pdf_keep_obj(ctx, pdf_array_get(ctx, p->structarray, bdc->mcid_num));
+	bdc->mcid_obj = pdf_keep_obj(ctx, pdf_lookup_mcid_in_mcids(ctx, bdc->mcid_num, p->structarray));
 	str = pdf_dict_get(ctx, bdc->mcid_obj, PDF_NAME(Alt));
 	if (str)
 		bdc->alt.utf8 = pdf_new_utf8_from_pdf_string_obj(ctx, str);
@@ -2706,7 +2745,7 @@ pdf_filter_BX(fz_context *ctx, pdf_processor *proc)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_BX)
 		p->chain->op_BX(ctx, p->chain);
 }
@@ -2716,7 +2755,7 @@ pdf_filter_EX(fz_context *ctx, pdf_processor *proc)
 {
 	pdf_sanitize_processor *p = (pdf_sanitize_processor*)proc;
 
-	filter_flush(ctx, p, 0);
+	filter_flush(ctx, p, FLUSH_OP);
 	if (p->chain->op_EX)
 		p->chain->op_EX(ctx, p->chain);
 }
