@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "mupdf/pdf.h"
 
@@ -34,7 +56,6 @@ pdf_pattern *
 pdf_load_pattern(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
 {
 	pdf_pattern *pat;
-	pdf_obj *obj;
 
 	if ((pat = pdf_find_item(ctx, pdf_drop_pattern_imp, dict)) != NULL)
 	{
@@ -53,20 +74,13 @@ pdf_load_pattern(fz_context *ctx, pdf_document *doc, pdf_obj *dict)
 		/* Store pattern now, to avoid possible recursion if objects refer back to this one */
 		pdf_store_item(ctx, dict, pat, pdf_pattern_size(pat));
 
-		pat->ismask = pdf_to_int(ctx, pdf_dict_get(ctx, dict, PDF_NAME_PaintType)) == 2;
-		pat->xstep = pdf_to_real(ctx, pdf_dict_get(ctx, dict, PDF_NAME_XStep));
-		pat->ystep = pdf_to_real(ctx, pdf_dict_get(ctx, dict, PDF_NAME_YStep));
+		pat->ismask = pdf_dict_get_int(ctx, dict, PDF_NAME(PaintType)) == 2;
+		pat->xstep = pdf_dict_get_real(ctx, dict, PDF_NAME(XStep));
+		pat->ystep = pdf_dict_get_real(ctx, dict, PDF_NAME(YStep));
+		pat->bbox = pdf_dict_get_rect(ctx, dict, PDF_NAME(BBox));
+		pat->matrix = pdf_dict_get_matrix(ctx, dict, PDF_NAME(Matrix));
 
-		obj = pdf_dict_get(ctx, dict, PDF_NAME_BBox);
-		pdf_to_rect(ctx, obj, &pat->bbox);
-
-		obj = pdf_dict_get(ctx, dict, PDF_NAME_Matrix);
-		if (obj)
-			pdf_to_matrix(ctx, obj, &pat->matrix);
-		else
-			pat->matrix = fz_identity;
-
-		pat->resources = pdf_dict_get(ctx, dict, PDF_NAME_Resources);
+		pat->resources = pdf_dict_get(ctx, dict, PDF_NAME(Resources));
 		if (pat->resources)
 			pdf_keep_obj(ctx, pat->resources);
 
