@@ -2081,7 +2081,7 @@ int pdfaudit_main(int argc, char **argv)
 	{
 		switch (c)
 		{
-		case 'o': outfile = fz_optarg; break;
+		case 'o': outfile = fz_optpath(fz_optarg); break;
 		case 0:
 		{
 			SWITCH(fz_optlong->opaque)
@@ -2116,18 +2116,13 @@ int pdfaudit_main(int argc, char **argv)
 
 	fz_try(ctx)
 	{
-		if (strcmp(outfile, "-") == 0)
-			out = fz_stdout(ctx);
-		else
-			out = fz_new_output_with_path(ctx, outfile, append);
+		out = fz_new_output_with_path(ctx, outfile, append);
 		while (fz_optind < argc)
 			filter_file(ctx, out, argv[fz_optind++]);
-		if (strcmp(outfile, "-") != 0)
-			fz_close_output(ctx, out);
+		fz_close_output(ctx, out);
 	}
 	fz_always(ctx)
-		if (strcmp(outfile, "-") != 0)
-			fz_drop_output(ctx, out);
+		fz_drop_output(ctx, out);
 	fz_catch(ctx)
 	{
 		fz_report_error(ctx);
