@@ -51,6 +51,7 @@ static int usage(void)
 		"\t-b\tprint only stream contents, as raw binary data\n"
 		"\t-g\tprint only object, one line per object, suitable for grep\n"
 		"\t-r\tforce repair before showing any objects\n"
+		"\t-L\tshow object labels\n"
 		"\tpath: path to an object, starting with either an object number,\n"
 		"\t\t'pages', 'trailer', or a property in the trailer;\n"
 		"\t\tpath elements separated by '.' or '/'. Path elements must be\n"
@@ -621,7 +622,7 @@ int pdfshow_main(int argc, char **argv)
 		switch (c)
 		{
 		case 'p': password = fz_optarg; break;
-		case 'o': output = fz_optarg; break;
+		case 'o': output = fz_optpath(fz_optarg); break;
 		case 'b': showbinary = 1; break;
 		case 'e': showdecode = 0; break;
 		case 'g': do_tight = 1; break;
