@@ -1,3 +1,25 @@
+// Copyright (C) 2004-2024 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 #include "xps-imp.h"
 
@@ -6,12 +28,16 @@
 #include <stdlib.h>
 
 static char *
-xps_parse_float_array(fz_context *ctx, xps_document *doc, char *s, int num, float *x)
+xps_parse_float_array(fz_context *ctx, xps_document *doc, char *s, int num, int *obtained, float *x)
 {
 	int k = 0;
 
 	if (s == NULL || *s == 0)
+	{
+		if (obtained)
+			*obtained = k;
 		return NULL;
+	}
 
 	while (*s)
 	{
@@ -25,6 +51,8 @@ xps_parse_float_array(fz_context *ctx, xps_document *doc, char *s, int num, floa
 		if (++k == num)
 			break;
 	}
+	if (obtained)
+		*obtained = k;
 	return s;
 }
 
@@ -33,10 +61,14 @@ xps_parse_point(fz_context *ctx, xps_document *doc, char *s_in, float *x, float
 {
 	char *s_out = s_in;
 	float xy[2];
+	int obtained = 0;
 
-	s_out = xps_parse_float_array(ctx, doc, s_out, 2, &xy[0]);
-	*x = xy[0];
-	*y = xy[1];
+	s_out = xps_parse_float_array(ctx, doc, s_out, 2, &obtained, &xy[0]);
+	if (obtained >= 2)
+	{
+		*x = xy[0];
+		*y = xy[1];
+	}
 	return s_out;
 }
 
@@ -213,12 +245,6 @@ xps_draw_arc(fz_context *ctx, xps_document *doc, fz_path *path,
 	fz_lineto(ctx, path, point_x, point_y);
 }
 
-/*
- * Parse an abbreviated geometry string, and call
- * ghostscript moveto/lineto/curveto functions to
- * build up a path.
- */
-
 fz_path *
 xps_parse_abbreviated_geometry(fz_context *ctx, xps_document *doc, char *geom, int *fill_rule)
 {
@@ -239,7 +265,7 @@ xps_parse_abbreviated_geometry(fz_context *ctx, xps_document *doc, char *geom, i
 
 	fz_try(ctx)
 	{
-		args = fz_malloc_array(ctx, strlen(geom) + 1, sizeof(char*));
+		args = fz_malloc_array(ctx, strlen(geom) + 1, char*);
 		pargs = args;
 
 		while (*s)
@@ -547,6 +573,7 @@ xps_parse_poly_quadratic_bezier_segment(fz_context *ctx, xps_document *doc, fz_p
 	while (*s != 0)
 	{
 		while (*s == ' ') s++;
+		x[n] = y[n] = 0;
 		s = xps_parse_point(ctx, doc, s, &x[n], &y[n]);
 		n ++;
 		if (n == 2)
@@ -595,6 +622,7 @@ xps_parse_poly_bezier_segment(fz_context *ctx, xps_document *doc, fz_path *path,
 	while (*s != 0)
 	{
 		while (*s == ' ') s++;
+		x[n] = y[n] = 0;
 		s = xps_parse_point(ctx, doc, s, &x[n], &y[n]);
 		n ++;
 		if (n == 3)
@@ -633,6 +661,7 @@ xps_parse_poly_line_segment(fz_context *ctx, xps_document *doc, fz_path *path, f
 	while (*s != 0)
 	{
 		while (*s == ' ') s++;
+		x = y = 0;
 		s = xps_parse_point(ctx, doc, s, &x, &y);
 		if (stroking && !is_stroked)
 			fz_moveto(ctx, path, x, y);
@@ -786,15 +815,14 @@ xps_clip(fz_context *ctx, xps_document *doc, fz_matrix ctm, xps_resource *dict,
 		path = xps_parse_path_geometry(ctx, doc, dict, clip_tag, 0, &fill_rule);
 	else
 		path = fz_new_path(ctx);
-	fz_clip_path(ctx, dev, path, fill_rule == 0, ctm, fz_infinite_rect);
-	fz_drop_path(ctx, path);
+	fz_try(ctx)
+		fz_clip_path(ctx, dev, path, fill_rule == 0, ctm, fz_infinite_rect);
+	fz_always(ctx)
+		fz_drop_path(ctx, path);
+	fz_catch(ctx)
+		fz_rethrow(ctx);
 }
 
-/*
- * Parse an XPS <Path> element, and call relevant ghostscript
- * functions for drawing and/or clipping the child elements.
- */
-
 void
 xps_parse_path(fz_context *ctx, xps_document *doc, fz_matrix ctm, char *base_uri, xps_resource *dict, fz_xml *root)
 {
@@ -1021,7 +1049,7 @@ xps_parse_path(fz_context *ctx, xps_document *doc, fz_matrix ctm, char *base_uri
 				samples[0] *= fz_atof(fill_opacity_att);
 			xps_set_color(ctx, doc, colorspace, samples);
 			fz_fill_path(ctx, dev, path, fill_rule == 0, ctm,
-				doc->colorspace, doc->color, doc->alpha, NULL);
+				doc->colorspace, doc->color, doc->alpha, fz_default_color_params);
 		}
 
 		if (fill_tag)
@@ -1038,7 +1066,7 @@ xps_parse_path(fz_context *ctx, xps_document *doc, fz_matrix ctm, char *base_uri
 				samples[0] *= fz_atof(stroke_opacity_att);
 			xps_set_color(ctx, doc, colorspace, samples);
 			fz_stroke_path(ctx, dev, stroke_path, stroke, ctm,
-				doc->colorspace, doc->color, doc->alpha, NULL);
+				doc->colorspace, doc->color, doc->alpha, fz_default_color_params);
 		}
 
 		if (stroke_tag)
