@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------------
 //
 //  Little Color Management System
-//  Copyright (c) 1998-2017 Marti Maria Saguer
+//  Copyright (c) 1998-2023 Marti Maria Saguer
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the "Software"),
@@ -96,43 +96,44 @@ void Help(void)
     fprintf(stderr, "usage: transicc [flags] [CGATS input] [CGATS output]\n\n");
 
     fprintf(stderr, "flags:\n\n");
-    fprintf(stderr, "%cv<0..3> - Verbosity level\n", SW);
+    fprintf(stderr, "-v<0..3> - Verbosity level\n");
 
-    fprintf(stderr, "%ce[op] - Encoded representation of numbers\n", SW);
-    fprintf(stderr, "\t%cw - use 16 bits\n", SW);
-    fprintf(stderr, "\t%cx - Hexadecimal\n\n", SW);
+    fprintf(stderr, "-e[op] - Encoded representation of numbers\n");
+    fprintf(stderr, "\t-w - use 16 bits\n");
+    fprintf(stderr, "\t-x - Hexadecimal\n\n");
 
-    fprintf(stderr, "%cs - bounded mode (clip negatives and highliths)\n", SW);
-    fprintf(stderr, "%cq - Quantize (round decimals)\n\n", SW);
+    fprintf(stderr, "-s - bounded mode (clip negatives and highlights)\n");
+    fprintf(stderr, "-q - Quantize (round decimals)\n\n");
 
-    fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
-    fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);
-    fprintf(stderr, "%cl<profile> - Transform by device-link profile\n", SW);
+    fprintf(stderr, "-i<profile> - Input profile (defaults to sRGB)\n");
+    fprintf(stderr, "-o<profile> - Output profile (defaults to sRGB)\n");
+    fprintf(stderr, "-l<profile> - Transform by device-link profile\n");
 
-    fprintf(stderr, "\nYou can use '*Lab', '*xyz' and others as built-in profiles\n\n");
+    PrintBuiltins();
 
-    PrintRenderingIntents();
+    PrintRenderingIntents(NULL);
 
     fprintf(stderr, "\n");
 
-    fprintf(stderr, "%cd<0..1> - Observer adaptation state (abs.col. only)\n\n", SW);
+    fprintf(stderr, "-d<0..1> - Observer adaptation state (abs.col. only)\n\n");
 
-    fprintf(stderr, "%cb - Black point compensation\n", SW);
+    fprintf(stderr, "-b - Black point compensation\n");
 
-    fprintf(stderr, "%cc<0,1,2,3> Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes)\n\n", SW);
-    fprintf(stderr, "%cn - Terse output, intended for pipe usage\n", SW);
+    fprintf(stderr, "-c<0,1,2,3> Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes)\n\n");
+    fprintf(stderr, "-n - Terse output, intended for pipe usage\n");
 
-    fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
-    fprintf(stderr, "%cm<0,1,2,3> - Soft proof intent\n", SW);
-    fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n\n", SW);
+    fprintf(stderr, "-p<profile> - Soft proof profile\n");
+    fprintf(stderr, "-m<0,1,2,3> - Soft proof intent\n");
+    fprintf(stderr, "-g - Marks out-of-gamut colors on softproof\n\n");
 
 
 
-    fprintf(stderr, "This program is intended to be a demo of the little cms\n"
-        "engine. Both lcms and this program are freeware. You can\n"
-        "obtain both in source code at http://www.littlecms.com\n"
+    fprintf(stderr, "This program is intended to be a demo of the Little CMS\n"
+        "color engine. Both lcms and this program are open source.\n"
+        "You can obtain both in source code at https://www.littlecms.com\n"
         "For suggestions, comments, bug reports etc. send mail to\n"
         "info@littlecms.com\n\n");
+
 }
 
 
@@ -140,15 +141,27 @@ void Help(void)
 // The toggles stuff
 
 static
-void HandleSwitches(int argc, char *argv[])
+void HandleSwitches(cmsContext ContextID, int argc, char *argv[])
 {
     int s;
 
     while ((s = xgetopt(argc, argv,
-        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqSsT:t:V:v:WwxX!:")) != EOF) {
+        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqSsT:t:V:v:WwxX!:-:")) != EOF) {
 
     switch (s){
 
+        case '-':
+            if (strcmp(xoptarg, "help") == 0)
+            {
+                Help();
+                exit(0);
+            }
+            else
+            {
+                FatalError("Unknown option - run without args to see valid ones.\n");
+            }
+            break;
+
         case '!':
             IncludePart = xoptarg;
             break;
@@ -172,7 +185,7 @@ void HandleSwitches(int argc, char *argv[])
                 ObserverAdaptationState > 1.0)
                 FatalError("Adaptation states should be between 0 and 1");
 
-            cmsSetAdaptationState(ObserverAdaptationState);
+            cmsSetAdaptationState(ContextID, ObserverAdaptationState);
                   }
                   break;
 
@@ -386,7 +399,7 @@ cmsNAMEDCOLORLIST* ComponentNames(cmsContext ContextID, cmsColorSpaceSignature s
 
         SetRange(1, IsInput);
 
-        n = cmsChannelsOf(ContextID, space);
+        n = cmsChannelsOfColorSpace(ContextID, space);
 
         for (i=0; i < n; i++) {
 
@@ -457,7 +470,7 @@ cmsBool OpenTransforms(cmsContext ContextID)
 
         if (cmsGetDeviceClass(ContextID, hInput) == cmsSigLinkClass ||
             cmsGetDeviceClass(ContextID, hOutput) == cmsSigLinkClass)
-            FatalError("Use %cl flag for devicelink profiles!\n", SW);
+            FatalError("Use -l flag for devicelink profiles!\n");
 
 
         InputColorSpace   = cmsGetColorSpace(ContextID, hInput);
@@ -561,13 +574,13 @@ cmsBool OpenTransforms(cmsContext ContextID)
         for (i=0; i < cmsMAXCHANNELS; i++)
             Alarm[i] = 0xFFFF;
 
-        cmsSetAlarmCodes(Alarm);
+        cmsSetAlarmCodes(ContextID, Alarm);
         dwFlags |= cmsFLAGS_GAMUTCHECK;
     }
 
 
     // The main transform
-    hTrans = cmsCreateProofingTransform(hInput,  dwIn, hOutput, dwOut, hProof, Intent, ProofingIntent, dwFlags);
+    hTrans = cmsCreateProofingTransform(ContextID, hInput,  dwIn, hOutput, dwOut, hProof, Intent, ProofingIntent, dwFlags);
 
     if (hProof) cmsCloseProfile(ContextID, hProof);
 
@@ -579,13 +592,13 @@ cmsBool OpenTransforms(cmsContext ContextID)
 
     if (hOutput && Verbose > 1) {
 
-        cmsHPROFILE hXYZ = cmsCreateXYZProfile();
-        cmsHPROFILE hLab = cmsCreateLab4Profile(NULL);
+        cmsHPROFILE hXYZ = cmsCreateXYZProfile(ContextID);
+        cmsHPROFILE hLab = cmsCreateLab4Profile(ContextID, NULL);
 
-        hTransXYZ = cmsCreateTransform(hInput, dwIn, hXYZ,  lIsFloat ? TYPE_XYZ_DBL : TYPE_XYZ_16, Intent, cmsFLAGS_NOCACHE);
+        hTransXYZ = cmsCreateTransform(ContextID, hInput, dwIn, hXYZ,  lIsFloat ? TYPE_XYZ_DBL : TYPE_XYZ_16, Intent, cmsFLAGS_NOCACHE);
         if (hTransXYZ == NULL) return FALSE;
 
-        hTransLab = cmsCreateTransform(hInput, dwIn, hLab,  lIsFloat? TYPE_Lab_DBL : TYPE_Lab_16, Intent, cmsFLAGS_NOCACHE);
+        hTransLab = cmsCreateTransform(ContextID, hInput, dwIn, hLab,  lIsFloat? TYPE_Lab_DBL : TYPE_Lab_16, Intent, cmsFLAGS_NOCACHE);
         if (hTransLab == NULL) return FALSE;
 
         cmsCloseProfile(ContextID, hXYZ);
@@ -652,7 +665,7 @@ void PrintFloatResults(cmsContext ContextID, cmsFloat64Number Value[])
     char ChannelName[cmsMAX_PATH];
     cmsFloat64Number v;
 
-    n = cmsChannelsOf(ContextID, OutputColorSpace);
+    n = cmsChannelsOfColorSpace(ContextID, OutputColorSpace);
     for (i=0; i < n; i++) {
 
         if (OutputColorant != NULL) {
@@ -732,7 +745,7 @@ void TakeFloatValues(cmsContext ContextID, cmsFloat64Number Float[])
         return;
     }
 
-    n = cmsChannelsOf(ContextID, InputColorSpace);
+    n = cmsChannelsOfColorSpace(ContextID, InputColorSpace);
     for (i=0; i < n; i++) {
 
         if (InputColorant) {
@@ -781,7 +794,7 @@ void PrintEncodedResults(cmsContext ContextID, cmsUInt16Number Encoded[])
     char ChannelName[cmsMAX_PATH];
     cmsUInt32Number v;
 
-    n = cmsChannelsOf(ContextID, OutputColorSpace);
+    n = cmsChannelsOfColorSpace(ContextID, OutputColorSpace);
     for (i=0; i < n; i++) {
 
         if (OutputColorant != NULL) {
@@ -858,7 +871,7 @@ cmsFloat64Number GetIT8Val(cmsContext ContextID, const char* Name, cmsFloat64Num
 // Read input values from CGATS file.
 
 static
-    void TakeCGATSValues(cmsContext ContextID, int nPatch, cmsFloat64Number Float[])
+void TakeCGATSValues(cmsContext ContextID, int nPatch, cmsFloat64Number Float[])
 {
 
     // At first take the name if SAMPLE_ID is present
@@ -947,7 +960,7 @@ static
         {
             cmsUInt32Number i, n;
 
-            n = cmsChannelsOf(ContextID, InputColorSpace);
+            n = cmsChannelsOfColorSpace(ContextID, InputColorSpace);
             for (i=0; i < n; i++) {
 
                 char Buffer[255];
@@ -963,7 +976,7 @@ static
         {
             cmsUInt32Number i, n;
 
-            n = cmsChannelsOf(ContextID, InputColorSpace);
+            n = cmsChannelsOfColorSpace(ContextID, InputColorSpace);
             for (i=0; i < n; i++) {
 
                 char Buffer[255];
@@ -1054,9 +1067,9 @@ void PutCGATSValues(cmsContext ContextID, cmsFloat64Number Float[])
     case cmsSig15colorData:
         {
 
-            cmsUInt32Number i, n;
+            cmsInt32Number i, n;
 
-            n = cmsChannelsOf(ContextID, InputColorSpace);
+            n = cmsChannelsOfColorSpace(ContextID, InputColorSpace);
             for (i=0; i < n; i++) {
 
                 char Buffer[255];
@@ -1071,9 +1084,9 @@ void PutCGATSValues(cmsContext ContextID, cmsFloat64Number Float[])
     default:
         {
 
-            cmsUInt32Number i, n;
+            cmsInt32Number i, n;
 
-            n = cmsChannelsOf(ContextID, InputColorSpace);
+            n = cmsChannelsOfColorSpace(ContextID, InputColorSpace);
             for (i=0; i < n; i++) {
 
                 char Buffer[255];
@@ -1174,7 +1187,7 @@ void SetOutputDataFormat(cmsContext ContextID)
             int i, n;
             char Buffer[255];
 
-            n = cmsChannelsOf(ContextID, OutputColorSpace);
+            n = cmsChannelsOfColorSpace(ContextID, OutputColorSpace);
             cmsIT8SetPropertyDbl(ContextID, hIT8out, "NUMBER_OF_FIELDS", n+1);
             cmsIT8SetDataFormat(ContextID, hIT8out, 0, "SAMPLE_ID");
 
@@ -1190,7 +1203,7 @@ void SetOutputDataFormat(cmsContext ContextID)
         int i, n;
         char Buffer[255];
 
-        n = cmsChannelsOf(ContextID, OutputColorSpace);
+        n = cmsChannelsOfColorSpace(ContextID, OutputColorSpace);
         cmsIT8SetPropertyDbl(ContextID, hIT8out, "NUMBER_OF_FIELDS", n+1);
         cmsIT8SetDataFormat(ContextID, hIT8out, 0, "SAMPLE_ID");
 
@@ -1241,9 +1254,11 @@ int main(int argc, char *argv[])
 
     int nPatch = 0;
 
-    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 4.3 [LittleCMS %2.2f]\n", LCMS_VERSION / 1000.0);
+    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 5.1 [LittleCMS %2.2f]\n", cmsGetEncodedCMMversion() / 1000.0);
+    fprintf(stderr, "Copyright (c) 1998-2023 Marti Maria Saguer. See COPYING file for details.\n");
+    fflush(stderr);
 
-    InitUtils("transicc");
+    InitUtils(ContextID, "transicc");
 
     Verbose = 1;
 
@@ -1253,7 +1268,7 @@ int main(int argc, char *argv[])
         return 0;
     }
 
-    HandleSwitches(argc, argv);
+    HandleSwitches(ContextID, argc, argv);
 
     // Open profiles, create transforms
     if (!OpenTransforms(ContextID)) return 1;
@@ -1313,5 +1328,3 @@ int main(int argc, char *argv[])
     // All is ok
     return 0;
 }
-
-
